@@ -71,8 +71,6 @@
 // namespace Funct
 // usual functions
 #include "../projects/Analysis/include/STK_Funct_Util.h"
-// raw functions for poisson function
-#include "../projects/Analysis/include/STK_Funct_poisson_raw.h"
 // raw functions
 #include "../projects/Analysis/include/STK_Funct_raw.h"
 // gamma function
