@@ -44,10 +44,6 @@
 #include "../projects/Rtkpp/include/STK_RVector.h"
 #include "../projects/Rtkpp/include/STK_RMatrix.h"
 
-#include "../projects/Rtkpp/include/STK_RDataHandler.h"
-#include "../projects/Rtkpp/include/STK_ClusterFacade.h"
-#include "../projects/Rtkpp/include/STK_ClusterLauncher.h"
-
 #include "../projects/Rtkpp/include/STK_wrap.h"
 
 #include "../projects/Rtkpp/include/Rcpp_wrap.h"
