@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2007  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -23,68 +23,77 @@
 */
 
 /*
- * Project:  Algebra
- * Purpose:  Implement the Qr Class.
- * Author:   Serge Iovleff, S..._Dot_I..._At_stkpp_Dot_org (see copyright for ...)
- *
+ * Project:  stkpp::Algebra
+ * created on: 20 nov. 2013
+ * Author:   iovleff, S..._Dot_I..._At_stkpp_Dot_org (see copyright for ...)
  **/
 
-/** @file STK_Qr.cpp
- *  @brief In this file we implement the Qr Class (QR decomposition).
+/** @file STK_lapack_Qr.h
+ *  @brief In this file we define the enclosing class of the dgeqrf lapack routine.
  **/
- 
+
 #include "../include/STK_lapack_Qr.h"
 
-#include "Arrays/include/STK_Array2DPoint.h"
-#include "Arrays/include/STK_CArray.h"
-
 namespace STK
 {
 
 namespace lapack
 {
-/* Computing the QR decomposition of the matrix Q_. */
-bool Qr::runImpl()
+
+/* private method for computing the Qr decomposition using a CArrayXX array */
+bool Qr::computeQr(CArrayXX& a, CVectorX& tau)
 {
   // start qr iterations
-  int lwork =-1, m = Q_.sizeRows(), n= Q_.sizeCols();
-  int info, size = std::min(m, n);
+  int lWork =-1, m = a.sizeRows(), n= a.sizeCols();
+  int info;
   Real iwork;
-  Real *p_work, *p_tau;
-  p_tau  = new Real[size];
-  CArrayXX a = Q_;
-  a.shift(0, 0);
+  Real *p_work;
+
   // get size for work
-  info = geqrf(m, n, a.p_data(), m, p_tau, &iwork, lwork);
+  info = geqrf(m, n, a.p_data(), m, tau.p_data(), &iwork, lWork);
   // check any error
   if (info!=0)
   {
-    delete[] p_tau;
     if (info>0)
-    { msg_error_ = STKERROR_NO_ARG(lapack::Qr::runImpl,internal error);
+    { this->msg_error_ = STKERROR_NO_ARG(lapack::Qr::computeQr,internal error);
       return false;
     }
-    msg_error_= STKERROR_1ARG(lapack::Qr::runImpl get,-info,error parameter);
+    this->msg_error_= STKERROR_1ARG(lapack::Qr::computeQr get,-info,error parameter);
     return false;
   }
   // create
-  lwork = (int)iwork;
-  p_work = new Real[lwork];
-  info = geqrf(m, n, a.p_data(), m, p_tau, p_work, lwork);
+  lWork = (int)iwork;
+  p_work = new Real[lWork];
+  info = geqrf(m, n, a.p_data(), m, tau.p_data(), p_work, lWork);
+  // release working set
+  delete[] p_work;
   // check any error
   if (info!=0)
   {
-    delete[] p_tau;
-    delete[] p_work;
     if (info>0)
-    { msg_error_ = STKERROR_NO_ARG(lapack::Qr::runImpl,internal error);
-      return false;
+    { this->msg_error_ = STKERROR_NO_ARG(lapack::Qr::computeQr,internal error);
+     return false;
     }
-    msg_error_= STKERROR_1ARG(lapack::Qr::runImpl get,-info,error parameter);
+    this->msg_error_= STKERROR_1ARG(lapack::Qr::computeQr get,-info,error parameter);
     return false;
   }
+  return true;
+}
+/* Computing the QR decomposition of the matrix Q_. */
+bool Qr::runImpl()
+{
+  int begin = Q_.beginRows(); // same first index checked at construction of QR
+  int end   = std::min(Q_.endRows(), Q_.endCols());
+  int size  = std::min(Q_.sizeRows(), Q_.sizeCols());
+  CVectorX tau(size);
+  CArrayXX a = Q_;
+  a.shift(0, 0);
+  tau.shift(0);
+  // compute results
+  if (!computeQr(a, tau)) { return false;};
   // get results
-  a.shift(Q_.beginRows(), Q_.beginCols());
+  a.shift(begin, begin);
+  tau.shift(begin);
   R_.resize(Q_.rows(), Q_.cols());
   for (int i=Q_.beginRows(); i< Q_.endRows(); ++i)
   {
@@ -92,23 +101,19 @@ bool Qr::runImpl()
     for (int j=Q_.beginCols(); j< jmax; ++j) { Q_(i,j) = a(i,j);}
     for (int j=i; j< Q_.endCols(); ++j)      { R_(i,j) = a(i,j);}
   }
-  for (int i=0, j=Q_.beginCols(); i< size; ++i, ++j) { Q_(j,j) = -p_tau[i];}
-  // release
-  delete[] p_work;
-  delete[] p_tau;
-
+  for (int j=Q_.beginCols(); j<end; ++j) { Q_(j,j) = -tau[j];}
   return true;
 }
 
-int Qr::geqrf(int m, int n, double* a, int lda, double* tau, double *work, int lwork)
+inline int Qr::geqrf(int m, int n, Real* a, int lda, Real* tau, Real *work, int lWork)
 {
   int info = 1;
 
 #ifdef STKUSELAPACK
 #ifdef STKREALAREFLOAT
-  sgeqrf_(&m, &n, a, &lda, tau, work, &lwork, &info);
+  sgeqrf_(&m, &n, a, &lda, tau, work, &lWork, &info);
 #else
-  dgeqrf_(&m, &n, a, &lda, tau, work, &lwork, &info);
+  dgeqrf_(&m, &n, a, &lda, tau, work, &lWork, &info);
 #endif
 #endif
 
