@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -36,10 +36,8 @@
 #ifndef STK_ALLOCATORBASE_H
 #define STK_ALLOCATORBASE_H
 
-#include "STKernel/include/STK_Range.h"
-
-#include "Sdk/include/STK_Macros.h"
-
+#include <STKernel/include/STK_Range.h>
+#include <Sdk/include/STK_Macros.h>
 #include "STK_IContainerRef.h"
 
 namespace STK
@@ -48,22 +46,23 @@ namespace STK
 /** @ingroup Arrays
  *  @brief Templated base class for all Allocator classes.
  *
- *  The AllocatorBase class is the base class of all Arrays
- *  stored in memory : it manages the main pointer on the data.
- *  It derive from the IContainerRef class as an array stored in
+ *  The AllocatorBase class is the base class of all Allocators string data
+ *  in memory. This class manages the main pointer on the data.
+ *  It derives from the IContainerRef class as an array stored in
  *  memory can always be wrapped in some way or be a wrapper of
  *  data stored in memory.
  * 
  *  This class can also be used as a concrete class.
- *
- *  The class @c Type can be any type of data that can be stored in memory.
+ *  @tparam Type can be any type of data that can be stored in memory.
  **/
-template<typename Type>
-class AllocatorBase : public IContainerRef
+template<typename Type, int Size>
+class AllocatorBase: public IContainerRef
 {
   public:
+    typedef TRange<Size> AllocRange;
+
     /** Default constructor. */
-    inline AllocatorBase() : IContainerRef(false)
+    AllocatorBase() : IContainerRef(false)
                            , p_data_(0)
                            , rangeData_()
     {}
@@ -79,18 +78,24 @@ class AllocatorBase : public IContainerRef
      *  @param T : the array to copy
      *  @param ref : is this a wrapper of T ?
      **/
-    inline AllocatorBase( AllocatorBase const& T, bool ref = false)
+    AllocatorBase( AllocatorBase const& T, bool ref = false)
                         : IContainerRef(ref)
                         , p_data_(ref ? T.p_data_: 0)
                         , rangeData_(T.rangeData_)
     {/* derived class have to copy the data if ref==false */}
+    template<int OtherSize>
+    inline AllocatorBase( AllocatorBase<Type, OtherSize> const& T, bool ref = false)
+                        : IContainerRef(ref)
+                        , p_data_(ref ? T.p_data(): 0)
+                        , rangeData_(T.rangeData())
+    {/* derived class have to copy the data if ref==false */}
     /** @brief Wrapper or copy constructor. It is set as protected as we don't
      *  know, how the end-user want to copy the data.
      *  @param q ptr to the data to wrap
      *  @param I range of the data wrapped
      *  @param ref is this a wrapper ?
      **/
-    inline AllocatorBase( Type* const& q, Range const& I, bool ref)
+    AllocatorBase( Type* const& q, Range const& I, bool ref)
                         : IContainerRef(ref)
                         , p_data_(ref ? q : 0)
                         , rangeData_(I)
@@ -102,7 +107,7 @@ class AllocatorBase : public IContainerRef
      *  @param size of the data to wrap
      *  @param ref is this a wrapper ?
      **/
-    inline AllocatorBase( Type* const& q, int size, bool ref)
+    AllocatorBase( Type* const& q, int size, bool ref)
                         : IContainerRef(ref)
                         , p_data_(ref ? q : 0)
                         , rangeData_(Range(0,size-1))
@@ -111,6 +116,8 @@ class AllocatorBase : public IContainerRef
     /** destructor. */
     ~AllocatorBase() { free(); }
 
+    /** @return the range of the data*/
+    AllocRange const& rangeData() const { return rangeData_;}
     /** @return the first index of the data. */
     inline int firstData() const { return rangeData_.begin();}
     /**@return the ending index of the data */
@@ -119,8 +126,6 @@ class AllocatorBase : public IContainerRef
     inline int lastData() const { return rangeData_.lastIdx();}
     /** @return the size of the data */
     inline int sizeData() const { return rangeData_.size();}
-    /** @return the range of the data*/
-    inline Range const& rangeData() const { return rangeData_;}
     /** @return a pointer on the constant data set*/
     inline Type* const& p_data() const { return p_data_;}
     /** @return a pointer on the data set */
@@ -151,10 +156,27 @@ class AllocatorBase : public IContainerRef
 #endif
       return p_data_[pos];
     }
+    /** @brief main ptr memory allocation.
+     *  @param I range of the data allocated
+     **/
+    void malloc( Range const& I);
+    /** @brief function for main ptr memory reallocation.
+     *
+     *  If the size requested is greater than the allocated size,
+     *  the Type stored are saved and copied using the operator=. the Type
+     *  class have to provide this operator.
+     *
+     *  If the size requested is lesser than the allocated size, only
+     *  the first elements fitting in the container are copied.
+     *  @param I range of the data to reserve
+     **/
+    void realloc( Range const& I);
+    /** function for main ptr memory deallocation. */
+    void free();
     /** exchange this with T.
      *  @param T the container to exchange with T
      **/
-    inline AllocatorBase& exchange(AllocatorBase &T)
+    AllocatorBase& exchange(AllocatorBase &T)
     {
       std::swap(p_data_, T.p_data_);
       std::swap(rangeData_, T.rangeData_);
@@ -164,20 +186,17 @@ class AllocatorBase : public IContainerRef
     /** swap two elements of the Allocator.
      *  @param pos1, pos2 the positions of the first and second element
      **/
-    inline void swap(int pos1, int pos2)
+    void swap(int pos1, int pos2)
     { std::swap(p_data_[pos1], p_data_[pos2]);}
     /** @brief copy the Allocator T by value.
      *  The memory is free and the Allocator T is physically copied in this.
-     *
-     *  @return a copy of this
-     *
      *  @param T the allocator to copy by value
+     *  @return a copy of this
      **/
-    inline AllocatorBase& copy( AllocatorBase const& T)
+    AllocatorBase& copy( AllocatorBase const& T)
     {
       // allocate memory if necessary
       malloc(T.rangeData_);
-      setRangeData(T.rangeData_);
       // copy values
       for (int pos= T.firstData(); pos < T.endData(); ++pos)
       { p_data_[pos] = T.p_data_[pos];}
@@ -194,10 +213,10 @@ class AllocatorBase : public IContainerRef
      *
      *  @param T the allocator to copy as reference
      **/
-    inline AllocatorBase& move( AllocatorBase const& T)
+    AllocatorBase& move( AllocatorBase const& T)
     {
       if (this == &T) return *this;
-      if (!this->isRef()) free();
+      free();
       setPtrData(T.p_data_, T.rangeData_, T.isRef());
       T.setRef(true);
       return *this;
@@ -217,93 +236,6 @@ class AllocatorBase : public IContainerRef
       // translate data
       decPtrData(inc);
     }
-    /** @brief main ptr memory allocation.
-     *
-     *  @note if @code I.begin() != baseIdx @endcode the allocated memory
-     *  have to be addressed using the formula
-     *  @code p_data_[I.begin()+i] @endcode.
-     *
-     *  @param I range of the data allocated
-     **/
-    void malloc( Range const& I)
-    {
-      if ((this->rangeData() == I)&&(p_data_)&&(!this->isRef())) return;
-      // free any existing data
-      free();
-      const int size = I.size();
-      // check size
-      if (size <= 0)
-      {
-        setPtrData(0, I, false);
-        this->setRef(false);
-        return;
-      }
-      // allocate memory
-      try
-      {
-        setPtrData(new Type[size], Range(0, size), false);
-        decPtrData(I.begin());
-      }
-      catch (std::bad_alloc const& error)  // if an alloc error occur
-      {
-        setDefault();
-        STKRUNTIME_ERROR_1ARG(AllocatorBase::malloc, I, memory allocation failed);
-      }
-      this->setRef(false);
-    }
-    /** @brief function for main ptr memory reallocation.
-     *
-     *  If the size requested is greater than the allocated size,
-     *  the Type stored are saved and copied using the operator=. the Type
-     *  class have to provide this operator.
-     *
-     *  If the size requested is lesser than the allocated size, only
-     *  the first elements fitting in the container are copied.
-     *  @param I range of the data to reserve
-     **/
-    void realloc( Range const& I)
-    {
-      if ((this->rangeData() == I)&&(p_data_)&&(!this->isRef())) return;
-      const int size = I.size(), inc= I.begin();
-      // check size
-      if (size <= 0)
-      {
-        free();
-        setPtrData(0, I, false);
-        return;
-      }
-      try
-      {
-        // allocate memory and apply increment
-        Type* p  = new Type[size];
-         p -= inc;
-        // no error: copy data
-        const int firstData = std::max(rangeData_.begin(), I.begin())
-                    , lastData  = std::min(rangeData_.lastIdx(), I.lastIdx());
-        for (int i = firstData; i<=lastData; ++i) { p[i] = p_data_[i];}
-        // liberate old memory
-        free();
-        // set default values
-        setPtrData(p, I, false);
-      }
-      catch (std::bad_alloc const& error)  // if an alloc error occur
-      {
-        STKRUNTIME_ERROR_1ARG(AllocatorBase::realloc, I, memory allocation failed);
-      }
-    }
-    /** function for main ptr memory deallocation. */
-    void free()
-    {
-      // nothing to do for ref
-      if (this->isRef()) return;
-      // if there is elts
-      if (p_data_)
-      {
-        incPtrData(firstData());  // translate
-        delete [] p_data_;        // erase
-        setDefault();             // set default values
-      }
-    }
     /** @brief Set the address and the range of the data.
      * This method is to be used when the memory have been allocated outside.
      * Allocated memory is not freed.
@@ -312,7 +244,7 @@ class AllocatorBase : public IContainerRef
      *  @param rangeData the range of the data
      *  @param ref is p_data_ a wrapper ?
      **/
-    inline void setPtrData( Type* p_data,  Range const& rangeData, bool ref)
+    void setPtrData( Type* p_data,  Range const& rangeData, bool ref)
     { p_data_ = p_data; rangeData_ = rangeData; this->setRef(ref);}
 
   private:
@@ -320,15 +252,15 @@ class AllocatorBase : public IContainerRef
      *  to the end-user.
      *  @param p_data the address to set
      **/
-    inline void setPtrData( Type* p_data = 0) { p_data_ = p_data;}
+    void setPtrData( Type* p_data = 0) { p_data_ = p_data;}
     /** Set the index of the first data : this method is not destined
      *  to the end-user.
      *  @param rangeData the range of the data to set
      **/
-    inline void setRangeData( Range const& rangeData = Range())
+    void setRangeData( Range const& rangeData = Range())
     { rangeData_ = rangeData;}
     /** Set array members to default values. */
-    inline void setDefault()
+    void setDefault()
     {
       setPtrData();
       setRangeData();
@@ -336,7 +268,7 @@ class AllocatorBase : public IContainerRef
     /** Increment the address of the data.
      *  @param inc the increment to apply
      **/
-    inline void incPtrData( int const& inc)
+    void incPtrData( int const& inc)
     {
       if (p_data_) { p_data_ += inc;}
       rangeData_.dec(inc);
@@ -344,7 +276,7 @@ class AllocatorBase : public IContainerRef
     /** Decrement the address of the data.
      *  @param dec the increment to apply
      **/
-    inline void decPtrData( int const& dec)
+    void decPtrData( int const& dec)
     {
       if (p_data_) { p_data_ -= dec;}
       rangeData_.inc(dec);
@@ -353,9 +285,81 @@ class AllocatorBase : public IContainerRef
     /** Main pointer on the data. */
     Type* p_data_;
     /** Range of the data */
-    Range rangeData_;
+    AllocRange rangeData_;
 };
 
+template<typename Type, int Size>
+void AllocatorBase<Type,Size>::malloc( Range const& I)
+{
+  {
+    if ((this->rangeData() == I)&&(p_data_)&&(!this->isRef())) return;
+    // free any existing data
+    free();
+    // check size
+    if (I.size() <= 0)
+    {
+      setPtrData(0, I, false);
+      this->setRef(false);
+      return;
+    }
+    // allocate memory
+    try
+    {
+      setPtrData(new Type[I.size()], Range(0, I.size()), false);
+      decPtrData(I.begin());
+    }
+    catch (std::bad_alloc const& error)
+    {
+      setDefault();
+      STKRUNTIME_ERROR_1ARG(AllocatorBase::malloc, I, memory allocation failed);
+    }
+    this->setRef(false);
+  }
+}
+
+template<typename Type, int Size>
+void AllocatorBase<Type,Size>::realloc( Range const& I)
+{
+  if ((this->rangeData() == I)&&(p_data_)&&(!this->isRef())) return;
+  // check size
+  if (I.size() <= 0)
+  {
+    free();
+    setPtrData(0, I, false);
+    return;
+  }
+  try
+  {
+    // allocate memory and apply increment
+    Type* p  = new Type[I.size()];
+     p -= I.begin();
+    // no error: copy data
+    const int begin = std::max(rangeData_.begin(), I.begin())
+            , end  = std::min(rangeData_.end(), I.end());
+    for (int i = begin; i<end; ++i) { p[i] = p_data_[i];}
+    // liberate old memory
+    free();
+    // set default values
+    setPtrData(p, I, false);
+  }
+  catch (std::bad_alloc const& error)
+  { STKRUNTIME_ERROR_1ARG(AllocatorBase::realloc, I, memory allocation failed);}
+}
+/** function for main ptr memory deallocation. */
+template<typename Type, int Size>
+void AllocatorBase<Type,Size>::free()
+{
+  // nothing to do for reference
+  if (this->isRef()) return;
+  // if there is elts
+  if (p_data_)
+  {
+    incPtrData(firstData());  // translate
+    delete [] p_data_;        // erase
+    setDefault();             // set default values
+  }
+}
+
 } // namespace STK
 
 #endif /* STK_ALLOCATORBASE_H */
