@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2012  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -40,7 +40,8 @@
 // this macro will be true if the assignation is correct and false otherwise
 #define CORRECT_ASSIGN(dst,src) \
 ( (    ( dst==Arrays::array2D_ || dst==Arrays::square_) \
-     &&( src==Arrays::array2D_ || src==Arrays::square_ || src==Arrays::diagonal_ || src==Arrays::lower_triangular_ || src==Arrays::upper_triangular_) \
+     &&( src==Arrays::array2D_ || src==Arrays::square_  \
+      || src==Arrays::diagonal_ || src==Arrays::lower_triangular_ || src==Arrays::upper_triangular_) \
   ) \
   ||( dst==Arrays::array2D_ && src==Arrays::vector_) \
   ||( dst==Arrays::array2D_ && src==Arrays::point_) \
@@ -66,58 +67,23 @@ namespace hidden
  *  */
 template < typename Derived, typename Rhs, int TStructure_, int RhsStructure_>
 struct Copycat;
+
 //---------------------GENERAL----------------------------------
 // general <- general
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::array2D_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
-  {
-    for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
-      for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
-      { lhs.elt(i, j) = rhs.elt(i, j);}
-  }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
     for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
-};
-
-// general <- vector
-template < typename Derived, typename Rhs>
-struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::vector_>
-{
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    int j = rhs.beginCols();
     for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
-    { lhs.elt(i, j) = rhs.elt(i, j);}
-  }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
-  {
-    int j = rhs.beginCols();
-    for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
-    { lhs.elt(i, j) = rhs.elt(i, j);}
-  }
-};
-
-// general <- point
-template < typename Derived, typename Rhs>
-struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::point_>
-{
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
-  {
-    int i = rhs.beginRows();
-    for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
-    { lhs.elt(i, j) = rhs.elt(i, j);}
-  }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
-  {
-    int i = rhs.beginRows();
-    for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
-    { lhs.elt(i, j) = rhs.elt(i, j);}
+      for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
+      { lhs.elt(i, j) = rhs.elt(i, j);}
   }
 };
 
@@ -125,15 +91,15 @@ struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::point_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::square_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
     for (int i = rhs.begin(); i< rhs.end(); ++i)
-      for (int j = rhs.begin(); j <= rhs.lastIdx(); ++j)
+      for (int j = rhs.begin(); j < rhs.end(); ++j)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    for (int j = rhs.begin(); j <= rhs.lastIdx(); ++j)
+    for (int j = rhs.begin(); j < rhs.lend(); ++j)
       for (int i = rhs.begin(); i< rhs.end(); ++i)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
@@ -144,12 +110,12 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::diagonal_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
     lhs.setValue(Type(0));
     for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i,i) = rhs.elt(i);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
     lhs.setValue(Type(0));
     for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i,i) = rhs.elt(i);}
@@ -161,27 +127,27 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::array2D_,   Arrays::lower_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
     { int i = rhs.beginRows();
-      for (; i < j; ++i)              { lhs.elt(i,j) = Type(0);}
+      for (; i < j; ++i)             { lhs.elt(i,j) = Type(0);}
       for (; i < rhs.endRows(); ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int j= last+1; j < rhs.endCols(); ++j)
+    for (int j= end; j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i) { lhs.elt(i,j) = Type(0);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i)
     { int j = rhs.beginCols();
-      for (; j <=i; ++j)              { lhs.elt(i,j) = rhs.elt(i, j);}
+      for (; j <=i; ++j)             { lhs.elt(i,j) = rhs.elt(i, j);}
       for (; j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
     }
-    for (int i= last+1; i < rhs.endRows(); ++i)
-      for (int j=1; j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
+    for (int i= end; i < rhs.endRows(); ++i)
+      for (int j=rhs.beginCols(); j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
 };
 
@@ -190,27 +156,63 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::upper_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
     { int i = rhs.beginRows();
-      for (; i <= j; ++i)             { lhs.elt(i,j) = rhs.elt(i, j);}
+      for (; i <= j; ++i)            { lhs.elt(i,j) = rhs.elt(i, j);}
       for (; i < rhs.endRows(); ++i) { lhs.elt(i,j) =  Type(0);}
     }
-    for (int j= last+1; j < rhs.endCols(); ++j)
+    for (int j= end; j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(lhs.lastIdxRows(), lhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i)
+    const int end = std::min(lhs.endRows(), lhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i)
     { int j = rhs.beginCols();
       for (; j <i; ++j)               { lhs.elt(i,j) = Type(0);}
       for (; j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int i= last+1; i < rhs.endRows(); ++i)
-      for (int j=1; j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
+    for (int i= end; i < rhs.endRows(); ++i)
+      for (int j=rhs.beginCols(); j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
+  }
+};
+
+// general <- vector
+template < typename Derived, typename Rhs>
+struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::vector_>
+{
+  static void runByCol(Derived& lhs, Rhs const& rhs )
+  {
+    int j = rhs.beginCols();
+    for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
+    { lhs.elt(i, j) = rhs.elt(i);}
+  }
+  static void runByRow(Derived& lhs, Rhs const& rhs )
+  {
+    int j = rhs.beginCols();
+    for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
+    { lhs.elt(i, j) = rhs.elt(i);}
+  }
+};
+
+// general <- point
+template < typename Derived, typename Rhs>
+struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::point_>
+{
+  static void runByCol(Derived& lhs, Rhs const& rhs )
+  {
+    int i = rhs.beginRows();
+    for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
+    { lhs.elt(i, j) = rhs.elt(j);}
+  }
+  static void runByRow(Derived& lhs, Rhs const& rhs )
+  {
+    int i = rhs.beginRows();
+    for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
+    { lhs.elt(i, j) = rhs.elt(j);}
   }
 };
 
@@ -219,13 +221,13 @@ struct Copycat<  Derived,  Rhs, Arrays::array2D_, Arrays::upper_triangular_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::square_, Arrays::array2D_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
     for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
       for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
     for (int j = rhs.beginCols(); j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i)
@@ -237,16 +239,16 @@ struct Copycat<  Derived,  Rhs, Arrays::square_, Arrays::array2D_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::square_, Arrays::square_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    for (int i = rhs.begin(); i< rhs.end(); ++i)
-      for (int j = rhs.begin(); j <= rhs.lastIdx(); ++j)
+    for (int j = rhs.begin(); j < rhs.end(); ++j)
+      for (int i = rhs.begin(); i< rhs.end(); ++i)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    for (int j = rhs.begin(); j <= rhs.lastIdx(); ++j)
-      for (int i = rhs.begin(); i< rhs.end(); ++i)
+    for (int i = rhs.begin(); i< rhs.end(); ++i)
+      for (int j = rhs.begin(); j < rhs.end(); ++j)
       { lhs.elt(i, j) = rhs.elt(i, j);}
   }
 };
@@ -256,17 +258,17 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::square_, Arrays::diagonal_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
     lhs.setValue(Type(0));
-    const int last = std::min(lhs.lastIdxRows(), lhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i) { lhs.elt(i,i) = rhs.elt(i);}
+    const int end = std::min(lhs.endRows(), lhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i) { lhs.elt(i,i) = rhs.elt(i);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
     lhs.setValue(Type(0));
-    const int last = std::min(lhs.lastIdxRows(), lhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i) { lhs.elt(i,i) = rhs.elt(i);}
+    const int end = std::min(lhs.endRows(), lhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i) { lhs.elt(i,i) = rhs.elt(i);}
   }
 };
 
@@ -275,27 +277,27 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::square_,   Arrays::lower_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
     { int i = rhs.beginRows();
       for (; i < j; ++i)                 { lhs.elt(i,j) = Type(0);}
       for (; i < rhs.endRows(); ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int j= last+1; j < rhs.endCols(); ++j)
+    for (int j= end; j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i) { lhs.elt(i,j) = Type(0);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i)
     { int j = rhs.beginCols();
       for (; j <=i; ++j)                 { lhs.elt(i,j) = rhs.elt(i, j);}
       for (; j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
     }
-    for (int i= last+1; i < rhs.endRows(); ++i)
-      for (int j=1; j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
+    for (int i= end; i < rhs.endRows(); ++i)
+      for (int j=rhs.beginCols(); j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
 };
 
@@ -304,27 +306,27 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::square_, Arrays::upper_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
     { int i = rhs.beginRows();
-      for (; i <= j; ++i)                { lhs.elt(i,j) = rhs.elt(i, j);}
+      for (; i <= j; ++i)            { lhs.elt(i,j) = rhs.elt(i, j);}
       for (; i < rhs.endRows(); ++i) { lhs.elt(i,j) =  Type(0);}
     }
-    for (int j= last+1; j < rhs.endCols(); ++j)
+    for (int j= end; j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(lhs.lastIdxRows(), lhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i)
+    const int end = std::min(lhs.endRows(), lhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i)
     { int j = rhs.beginCols();
       for (; j <i; ++j)                  { lhs.elt(i,j) = Type(0);}
       for (; j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int i= last+1; i < rhs.endRows(); ++i)
-      for (int j=1; j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
+    for (int i= end; i < rhs.endRows(); ++i)
+      for (int j=rhs.beginCols(); j < rhs.endCols(); ++j) { lhs.elt(i,j) = Type(0);}
   }
 };
 
@@ -334,22 +336,22 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::lower_triangular_, Arrays::lower_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
       for (int i=j; i < rhs.endRows(); ++i)
       { lhs.elt(i,j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int i = rhs.beginRows(); i <= last; ++i)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int i = rhs.beginRows(); i < end; ++i)
     {
       for (int j = rhs.beginCols(); j <=i; ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int i= last+1; i < rhs.endRows(); ++i)
-      for (int j=1; j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
+    for (int i= end; i < rhs.endRows(); ++i)
+      for (int j=rhs.beginCols(); j < rhs.endCols(); ++j) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
 };
 
@@ -359,17 +361,17 @@ template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::upper_triangular_, Arrays::upper_triangular_>
 {
   typedef typename hidden::Traits<Derived>::Type Type;
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   {
-    const int last = std::min(rhs.lastIdxRows(), rhs.lastIdxCols());
-    for (int j = rhs.beginCols(); j <= last; ++j)
+    const int end = std::min(rhs.endRows(), rhs.endCols());
+    for (int j = rhs.beginCols(); j < end; ++j)
     {
       for (int i = rhs.beginRows(); i <= j; ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
     }
-    for (int j= last+1; j < rhs.endCols(); ++j)
+    for (int j= end; j < rhs.endCols(); ++j)
       for (int i = rhs.beginRows(); i < rhs.endRows(); ++i) { lhs.elt(i,j) = rhs.elt(i, j);}
   }
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   {
     const int last = std::min(lhs.lastIdxRows(), lhs.lastIdxCols());
     for (int i = rhs.beginRows(); i <= last; ++i)
@@ -383,9 +385,9 @@ struct Copycat<  Derived,  Rhs, Arrays::upper_triangular_, Arrays::upper_triangu
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::diagonal_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -393,9 +395,9 @@ struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::diagonal_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::vector_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -403,9 +405,9 @@ struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::vector_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::point_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -414,9 +416,9 @@ struct Copycat<  Derived,  Rhs, Arrays::diagonal_, Arrays::point_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::vector_, Arrays::diagonal_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -424,18 +426,18 @@ struct Copycat<  Derived,  Rhs, Arrays::vector_, Arrays::diagonal_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::vector_, Arrays::vector_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 // vector <- point
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::vector_, Arrays::point_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -444,9 +446,9 @@ struct Copycat<  Derived,  Rhs, Arrays::vector_, Arrays::point_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::point_, Arrays::diagonal_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -454,18 +456,18 @@ struct Copycat<  Derived,  Rhs, Arrays::point_, Arrays::diagonal_>
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::point_, Arrays::vector_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 // vector <- point
 template < typename Derived, typename Rhs>
 struct Copycat<  Derived,  Rhs, Arrays::point_, Arrays::point_>
 {
-  inline static void runByCol(Derived& lhs, Rhs const& rhs )
+  static void runByCol(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
-  inline static void runByRow(Derived& lhs, Rhs const& rhs )
+  static void runByRow(Derived& lhs, Rhs const& rhs )
   { for (int i = rhs.begin(); i< rhs.end(); ++i) { lhs.elt(i) = rhs.elt(i);}}
 };
 
@@ -517,11 +519,6 @@ struct CopycatSelector< Derived, Rhs, Arrays::by_row_>
   { Copycat<Derived, Rhs, tstructure_, sstructure_>::runByRow(lhs, rhs );}
 };
 
-} // namespace hidden
-
-namespace hidden
-{
-
 /** @ingroup hidden
  * utility class that select the resize method to call
  **/
@@ -532,53 +529,64 @@ struct resizeSelector;
 template< typename Lhs, typename Rhs, int TStructure_>
 struct resizeSelector
 {
-  static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
   { dst.resize(src.rows(), src.cols());}
 };
+/** specialization for the square_ case */
+template< typename Lhs, typename Rhs>
+struct resizeSelector<Lhs, Rhs, Arrays::square_>
+{
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  { dst.resize(src.range());}
+};
 /** specialization for the diagonal_ case */
 template< typename Lhs, typename Rhs>
 struct resizeSelector<Lhs, Rhs, Arrays::diagonal_>
 {
-  static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
   { dst.resize(src.range());}
 };
 /** specialization for the vector_ case */
 template< typename Lhs, typename Rhs>
 struct resizeSelector<Lhs, Rhs, Arrays::vector_>
 {
-  static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
   { dst.resize(src.range());}
 };
 /** specialization for the point_ case */
 template< typename Lhs, typename Rhs>
 struct resizeSelector<Lhs, Rhs, Arrays::point_>
 {
-  static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
   { dst.resize(src.range());}
 };
 
+/** specialization for the number_ case */
+template< typename Lhs, typename Rhs>
+struct resizeSelector<Lhs, Rhs, Arrays::number_>
+{
+  inline static void run(Lhs& dst, ExprBase<Rhs> const& src )
+  { /* nothing to do */;}
+};
+
 }  // namespace hidden
 
 /* @brief assign src to this
  **/
 template<class Derived>
 template<class Rhs>
-Derived& ArrayBase<Derived>::assign(ExprBase<Rhs> const& rhs)
+inline Derived& ArrayBase<Derived>::assign(ExprBase<Rhs> const& rhs)
 {
-    enum
-    {
-      lhs_struct_ = hidden::Traits<Derived>::structure_
-    , lhs_orient_ = hidden::Traits<Derived>::orient_
-    , lhs_sizeRows_ = hidden::Traits<Derived>::sizeRows_
-    , lhs_sizeCols_ = hidden::Traits<Derived>::sizeCols_
-    , rhs_struct_ = hidden::Traits<Rhs>::structure_
-    , rhs_orient_ = hidden::Traits<Rhs>::orient_
-    , rhs_sizeRows_ = hidden::Traits<Rhs>::sizeRows_
-    , rhs_sizeCols_ = hidden::Traits<Rhs>::sizeCols_
-    };
-    STK_STATICASSERT(CORRECT_ASSIGN((Arrays::Structure)lhs_struct_, (Arrays::Structure)rhs_struct_),YOU_TRIED_TO_ASSIGN_A_NOT_COMPATIBLE_ARRAY);
+  enum
+  {
+    rhs_structure_ = hidden::Traits<Rhs>::structure_
+  , rhs_orient_ = hidden::Traits<Rhs>::orient_
+  , rhs_sizeRows_ = hidden::Traits<Rhs>::sizeRows_
+  , rhs_sizeCols_ = hidden::Traits<Rhs>::sizeCols_
+  };
+   STK_STATIC_ASSERT(CORRECT_ASSIGN((Arrays::Structure)structure_, (Arrays::Structure)rhs_structure_),YOU_TRIED_TO_ASSIGN_A_NOT_COMPATIBLE_ARRAY);
   // choose the correct way to resize if necessary
-  hidden::resizeSelector<Derived, Rhs, lhs_struct_>::run(this->asDerived(), rhs.asDerived());
+  hidden::resizeSelector<Derived, Rhs, rhs_structure_>::run(this->asDerived(), rhs.asDerived());
   // choose the correct way to copy
   hidden::CopycatSelector<Derived, Rhs,  orient_>::run(this->asDerived(), rhs.asDerived());
   return this->asDerived();
@@ -587,62 +595,62 @@ Derived& ArrayBase<Derived>::assign(ExprBase<Rhs> const& rhs)
 /* Adding a Rhs to this. */
 template<class Derived>
 template<typename Rhs>
-Derived&  ArrayBase<Derived>::operator+=( ExprBase<Rhs> const& other)
+inline Derived&  ArrayBase<Derived>::operator+=( ExprBase<Rhs> const& rhs)
 {
-  this->asDerived() = this->asDerived() + other;
+  this->asDerived() = this->asDerived() + rhs;
   return this->asDerived();
 }
 /* subtract a Rhs to this. */
 template<class Derived>
 template<typename Rhs>
-Derived&  ArrayBase<Derived>::operator-=( ExprBase<Rhs> const& other)
+inline Derived&  ArrayBase<Derived>::operator-=( ExprBase<Rhs> const& rhs)
 {
-  this->asDerived() = this->asDerived() - other;
+  this->asDerived() = this->asDerived() - rhs;
   return this->asDerived();
 }
 
 template<class Derived>
 template<typename Rhs>
-Derived&  ArrayBase<Derived>::operator/=( ExprBase<Rhs> const& other)
+inline Derived&  ArrayBase<Derived>::operator/=( ExprBase<Rhs> const& rhs)
 {
-  this->asDerived() = this->asDerived() / other;
+  this->asDerived() = this->asDerived() / rhs;
   return this->asDerived();
 }
-/* subtract a Rhs to this. */
+/* mult a Rhs to this. */
 template<class Derived>
 template<typename Rhs>
-Derived&  ArrayBase<Derived>::operator*=( ExprBase<Rhs> const& other)
+inline Derived&  ArrayBase<Derived>::operator*=( ExprBase<Rhs> const& rhs)
 {
-  this->asDerived() = this->asDerived() * other;
+  this->asDerived() = this->asDerived() * rhs;
   return this->asDerived();
 }
 
 /* Adding a constant to this. */
 template<class Derived>
-Derived& ArrayBase<Derived>::operator+=( Type const& other)
+inline Derived& ArrayBase<Derived>::operator+=( Type const& value)
 {
-  this->asDerived() = this->asDerived() + other;
+  this->asDerived() = this->asDerived() + value;
   return this->asDerived();
 }
 /* Substract a constant to this. */
 template<class Derived>
-Derived& ArrayBase<Derived>::operator-=( Type const& other)
+inline Derived& ArrayBase<Derived>::operator-=( Type const& value)
 {
-  this->asDerived() = this->asDerived() - other;
+  this->asDerived() = this->asDerived() - value;
   return this->asDerived();
 }
 /* product of this by a constant. */
 template<class Derived>
-Derived& ArrayBase<Derived>::operator*=( Type const& other)
+inline Derived& ArrayBase<Derived>::operator*=( Type const& value)
 {
-  this->asDerived() = this->asDerived() * other;
+  this->asDerived() = this->asDerived() * value;
   return this->asDerived();
 }
 /* dividing this by a constant. */
 template<class Derived>
-Derived& ArrayBase<Derived>::operator/=( Type const& other)
+inline Derived& ArrayBase<Derived>::operator/=( Type const& value)
 {
-  this->asDerived() = this->asDerived() / other;
+  this->asDerived() = this->asDerived() / value;
   return this->asDerived();
 }
 
