@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2011  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
@@ -77,8 +77,10 @@ template<typename Type_, int SizeCols_, bool Orient_>
 struct Traits< CArrayPoint<Type_, SizeCols_, Orient_> >
 {
     typedef CArrayNumber<Type_, Orient_> Number;
-    typedef CArrayNumber<Type_, Orient_> Col;
-    typedef CArrayNumber<Type_, Orient_> SubCol;
+//    typedef CArrayNumber<Type_, Orient_> Col;
+//    typedef CArrayNumber<Type_, Orient_> SubCol;
+    typedef CArrayVector<Type_, 1, Orient_> Col;
+    typedef CArrayVector<Type_, 1, Orient_> SubCol;
 
     typedef CArrayPoint<Type_, SizeCols_, Orient_> Row;
     typedef CArrayPoint<Type_, UnknownSize, Orient_> SubRow;
@@ -87,10 +89,10 @@ struct Traits< CArrayPoint<Type_, SizeCols_, Orient_> >
     typedef typename If<(SizeCols_ == 1), Number, SubRow>::Result SubVector;
     typedef typename If<(SizeCols_ == 1), Number, SubRow>::Result SubArray;
     // The CAllocator have to have the same structure than the CArray
-    typedef CAllocator<Type_, Arrays::point_, 1, SizeCols_, Orient_> Allocator;
-
-    typedef Type_ Type;
+    typedef CAllocator<Type_, 1, SizeCols_, Orient_> Allocator;
 
+    typedef Type_                Type;
+    typedef typename RemoveConst<Type_>::Type const& ReturnType;
     enum
     {
       structure_ = Arrays::point_,
@@ -107,12 +109,16 @@ struct Traits< CArrayPoint<Type_, SizeCols_, Orient_> >
 /** @ingroup Arrays
  * @brief declaration of  the point case.
  */
-template <typename Type, int SizeCols_, bool Orient_>
-class CArrayPoint : public ICArray < CArrayPoint<Type, SizeCols_, Orient_> >
+template <typename Type_, int SizeCols_, bool Orient_>
+class CArrayPoint : public ICArray < CArrayPoint<Type_, SizeCols_, Orient_> >
 {
   public:
-    typedef ICArray < CArrayPoint<Type, SizeCols_, Orient_> > Base;
-    typedef ArrayBase < CArrayPoint<Type, SizeCols_, Orient_> > LowBase;
+    typedef ICArray < CArrayPoint<Type_, SizeCols_, Orient_> > Base;
+    typedef ArrayBase < CArrayPoint<Type_, SizeCols_, Orient_> > LowBase;
+
+    typedef typename hidden::Traits<CArrayPoint<Type_, SizeCols_, Orient_> >::Type Type;
+    typedef typename hidden::Traits<CArrayPoint<Type_, SizeCols_, Orient_> >::ReturnType ReturnType;
+
     enum
     {
       structure_ = Arrays::point_,
@@ -123,54 +129,79 @@ class CArrayPoint : public ICArray < CArrayPoint<Type, SizeCols_, Orient_> >
     };
 
     /** Default constructor. */
-    inline CArrayPoint(): Base() {}
+    CArrayPoint(): Base() {}
     /** constructor with specified dimension.
      *  @param sizeCols range of the columns
      **/
-    inline CArrayPoint( int sizeCols): Base(1, sizeCols) {}
-    /** constructor with rbeg, rend, cbeg and cend specified,
-     *  initialization with a constant.
-     *  @param sizeCols range of the columns
+    CArrayPoint( int sizeCols): Base(1, sizeCols) {}
+    /** constructor with specified ranges.
+     *  @param range range of the rows and columns
+     **/
+    CArrayPoint( Range range): Base(1, range.size())
+    { this->shift(range.begin());}
+    /** constructor with specified size, initialization with a constant.
+     *  @param sizeCols size of the columns
      *  @param v initial value of the container
      **/
-    inline CArrayPoint( int sizeCols, Type const& v): Base(1, sizeCols, v) {}
+    CArrayPoint( int sizeCols, Type const& v): Base(1, sizeCols, v) {}
+    /** constructor with specified ranges, initialization with a constant.
+     *  @param range range of the columns
+     *  @param v initial value of the container
+     **/
+    CArrayPoint( Range range, Type const& v): Base(1, range.size(), v)
+    { this->shift(range.begin());}
     /** Copy constructor
      *  @param T the container to copy
      *  @param ref true if T is wrapped
      **/
-    inline CArrayPoint( CArrayPoint const& T, bool ref=false): Base(T, ref) {}
+    CArrayPoint( CArrayPoint const& T, bool ref=false): Base(T, ref) {}
+    /** constructor by reference, ref_=1.
+     *  @param T the container to wrap
+     *  @param J the columns range to wrap
+     **/
+    CArrayPoint( CArrayPoint const& T, Range const& J)
+               : Base(T, T.rows(), J) {}
+    /** constructor by reference, ref_=1.
+     *  @param T the container to wrap
+     *  @param J the range of the data to wrap
+     *  @param row the index of the row to wrap
+     **/
+    template<class OtherArray>
+    CArrayPoint( ICArray<OtherArray> const& T, Range const& J, int row)
+               : Base(T.allocator(), Range(row, 1), J)
+    {}
     /** wrapper constructor for 0 based C-Array.
      *  @param q pointer on the array
      *  @param nbCol number of columns
      **/
-    inline CArrayPoint( Type* const& q, int nbCol): Base(q, 1, nbCol) {}
+    CArrayPoint( Type* const& q, int nbCol): Base(q, 1, nbCol) {}
     /** constructor by reference.
      *  @param allocator the allocator to wrap
      **/
     template<class OtherAllocator>
-    inline CArrayPoint( CAllocatorBase<OtherAllocator> const& allocator)
+    CArrayPoint( ITContainer2D<OtherAllocator> const& allocator)
                       : Base(allocator.asDerived())
     {}
     /** Copy constructor using an expression.
      *  @param T the container to wrap
      **/
     template<class OtherDerived>
-    inline CArrayPoint( ExprBase<OtherDerived> const& T): Base(1,T.size()) { LowBase::operator=(T);}
+    CArrayPoint( ExprBase<OtherDerived> const& T): Base(1,T.size()) { LowBase::operator=(T);}
     /** destructor. */
-    inline ~CArrayPoint() {}
+    ~CArrayPoint() {}
     /** operator= : set the container to a constant value.
      *  @param v the value to set
      **/
-    inline CArrayPoint& operator=(Type const& v) { return LowBase::setValue(v);}
+    CArrayPoint& operator=(Type const& v) { return LowBase::setValue(v);}
     /** operator = : overwrite the CArrayPoint with the Right hand side T.
      *  @param T the container to copy
      **/
     template<class Rhs>
-    inline CArrayPoint& operator=( ExprBase<Rhs> const& T) { return LowBase::assign(T);}
+    CArrayPoint& operator=( ExprBase<Rhs> const& T) { return LowBase::assign(T);}
     /** operator = : overwrite the CArray with the Right hand side rhs.
      *  @param rhs the container to copy
      **/
-    inline CArrayPoint& operator=(CArrayPoint rhs) { return LowBase::assign(rhs);}
+    CArrayPoint& operator=(CArrayPoint rhs) { return LowBase::assign(rhs);}
 };
 
 } // namespace STK
