@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2012  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -39,7 +39,6 @@ namespace STK
 {
 
 // forward declaration
-template< typename Derived> class SlicingOperatorBase;
 template< typename Array> class RowOperator;
 template< typename Array> class ColOperator;
 // only for vectors, points and the like
@@ -72,15 +71,6 @@ template<> struct RowTraits<Arrays::vector_>
 template<> struct RowTraits<Arrays::number_>
 { enum { structure_ = Arrays::number_}; };
 
-} // end namespace hidden
-
-// forward declaration
-template< typename Lhs> class RowOperatorBase;
-template< typename Lhs> class ColOperatorBase;
-
-
-namespace hidden
-{
 /** @ingroup hidden
  *  @brief Traits class for the row operator
  */
@@ -115,11 +105,12 @@ struct Traits< RowOperator <Lhs> >
   * don't have to name RowOperator type explicitly.
   */
 template< typename Lhs>
-class RowOperator: public RowOperatorBase< Lhs>, public TRef<1>
+class RowOperator: public ExprBase< RowOperator< Lhs> >, public TRef<1>
 {
   public:
-    typedef RowOperatorBase< Lhs> Base;
-    typedef typename hidden::Traits< RowOperator<Lhs> >::Type Type;
+    typedef ExprBase< RowOperator< Lhs> > Base;
+    typedef typename hidden::Traits< RowOperator< Lhs> >::Type Type;
+    typedef typename hidden::Traits< RowOperator< Lhs> >::ReturnType ReturnType;
     enum
     {
         structure_ = hidden::Traits< RowOperator<Lhs> >::structure_,
@@ -135,47 +126,16 @@ class RowOperator: public RowOperatorBase< Lhs>, public TRef<1>
 
     /** Constructor */
     inline RowOperator( Lhs const& lhs, int i)
-                      : Base(i), lhs_(lhs), rows_(i,1), cols_(lhs_.rangeColsInRow(i)) {}
+                      : Base(), i_(i), lhs_(lhs), rows_(i,1), cols_(lhs_.rangeColsInRow(i)) {}
+    /** Copy constructor */
+    inline RowOperator( RowOperator const& row, bool ref)
+                      : Base(row), i_(row.i_), lhs_(row.lhs_), rows_(row.rows_), cols_(row.cols_) {}
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return rows_;}
-    /** @return the first index of the rows */
-    inline int beginRowsImpl() const { return rows_.begin();}
-    /** @return the ending index of the rows */
-    inline int endRowsImpl() const { return rows_.end();}
-    /** @return the number of rows */
-    inline int sizeRowsImpl() const { return 1;}
-
     /** @return the range of the Columns */
     inline ColRange const& colsImpl() const { return cols_;}
-    /** @return the first index of the columns */
-    inline int beginColsImpl() const { return cols_.begin();}
-    /** @return the ending index of the columns */
-    inline int endColsImpl() const { return cols_.end();}
-    /** @return the number of columns */
-    inline int sizeColsImpl() const { return cols_.size();}
-
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
-
-  protected:
-    Lhs const& lhs_;
-    RowRange rows_;
-    ColRange cols_;
-};
-
-/** @ingroup Arrays
-  * @brief implement the access to the elements in the general case.
-  **/
-template< typename Lhs>
-class RowOperatorBase: public ExprBase< RowOperator< Lhs> >
-{
-  public:
-    typedef RowOperator<Lhs> Derived;
-    typedef ExprBase< Derived > Base;
-    typedef typename hidden::Traits< Derived >::Type Type;
-    typedef typename hidden::Traits< Derived >::ReturnType ReturnType;
-    /** constructor. */
-    inline RowOperatorBase(int i) : Base(), i_(i) {}
     /** @return the element (i,j)
      *  @param i, j index of the row and column
      **/
@@ -190,15 +150,18 @@ class RowOperatorBase: public ExprBase< RowOperator< Lhs> >
     /** @return the element jth element
      *  @param j index of the jth element
      **/
-    inline ReturnType elt1Impl(int j) const
-    { return (this->asDerived().lhs().elt(i_, j));}
+    inline ReturnType elt1Impl(int j) const { return (this->asDerived().lhs().elt(i_, j));}
     /** accesses to the element */
-    inline ReturnType elt0Impl() const
-    { return (this->asDerived().lhs().elt());}
+    inline ReturnType elt0Impl() const { return (this->asDerived().lhs().elt());}
+
   protected:
     int i_;
+    Lhs const& lhs_;
+    RowRange rows_;
+    ColRange cols_;
 };
 
+
 namespace hidden
 {
 /** @ingroup hidden
@@ -236,11 +199,12 @@ struct Traits< ColOperator <Lhs> >
   * don't have to name ColOperator type explicitly.
   */
 template< typename Lhs>
-class ColOperator: public ColOperatorBase< Lhs>, public TRef<1>
+class ColOperator: public ExprBase< ColOperator< Lhs> >, public TRef<1>
 {
   public:
-    typedef ColOperatorBase< Lhs> Base;
+    typedef ExprBase< ColOperator< Lhs> > Base;
     typedef typename hidden::Traits< ColOperator<Lhs> >::Type Type;
+    typedef typename hidden::Traits< ColOperator<Lhs> >::ReturnType ReturnType;
     enum
     {
         structure_ = hidden::Traits< ColOperator<Lhs> >::structure_,
@@ -256,49 +220,19 @@ class ColOperator: public ColOperatorBase< Lhs>, public TRef<1>
 
     /** Constructor */
     inline ColOperator( Lhs const& lhs, int j)
-                      : Base(j), lhs_(lhs), rows_(lhs_.rangeRowsInCol(j)), cols_(j,1) {}
+                      : Base(), j_(j), lhs_(lhs), rows_(lhs_.rangeRowsInCol(j)), cols_(j,1) {}
+    /** Copy constructor */
+    inline ColOperator( ColOperator const& col, bool ref)
+                      : Base(col), j_(col.j_), lhs_(col.lhs_), rows_(col.rows_), cols_(col.cols_) {}
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return rows_;}
-    /** @return the first index of the rows */
-    inline int beginRowsImpl() const { return rows_.begin();}
-    /** @return the ending index of the rows */
-    inline int endRowsImpl() const { return rows_.end();}
-    /** @return the number of rows */
-    inline int sizeRowsImpl() const { return rows_.size();}
-
-    /** @return the range of the columns */
+    /** @return the columns range */
     inline ColRange const& colsImpl() const { return cols_;}
-    /** @return the first index of the columns */
-    inline int beginColsImpl() const { return cols_.begin();}
-    /** @return the ending index of the columns */
-    inline int endColsImpl() const { return cols_.end();}
-    /** @return the number of columns */
-    inline int sizeColsImpl() const { return 1;}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
-
-  protected:
-    Lhs const& lhs_;
-    RowRange rows_;
-    ColRange cols_;
-};
-
-/** @ingroup Arrays
-  * @brief implement the access to the elements in the general case.
-  **/
-template< typename Lhs>
-class ColOperatorBase : public ExprBase< ColOperator< Lhs> >
-{
-  public:
-    typedef ColOperator<Lhs> Derived;
-    typedef ExprBase< Derived > Base;
-    typedef typename hidden::Traits< Derived >::Type Type;
-    typedef typename hidden::Traits< Derived >::ReturnType ReturnType;
-    /** constructor. */
-    inline ColOperatorBase(int j) : Base(), j_(j) {}
-    /** @return the element (i,j)
-     *  @param i, j indexes of the row and column
+    /** @return the ith element
+     *  @param i,j indexes of the element
      **/
     inline ReturnType elt2Impl(int i, int j) const
     {
@@ -306,20 +240,23 @@ class ColOperatorBase : public ExprBase< ColOperator< Lhs> >
       if (j != j_)
       { STKRUNTIME_ERROR_2ARG(ColOperatorBase::elt2Impl,i,j,column index is not valid);}
 #endif
-      return (this->asDerived().lhs().elt(i, j_));
+      return (lhs_.elt(i, j_));
     }
     /** @return the element ith element
      *  @param i index of the ith element
      **/
-    inline ReturnType elt1Impl(int i) const
-    { return (this->asDerived().lhs().elt(i, j_));}
-    /** accesses to the element */
-    inline ReturnType elt0Impl() const
-    { return (this->asDerived().lhs().elt());}
+    inline ReturnType elt1Impl(int i) const { return (lhs_.elt(i, j_));}
+    /** access to the element */
+    inline ReturnType elt0Impl() const { return (lhs_.elt(j_));}
+
   protected:
     int j_;
+    Lhs const& lhs_;
+    RowRange rows_;
+    ColRange cols_;
 };
 
+
 namespace hidden
 {
 /** @ingroup hidden
@@ -332,8 +269,8 @@ struct Traits< SubOperator <Lhs> >
   {
     structure_ = Lhs::structure_,
     orient_    = Lhs::orient_,
-    sizeRows_  = Lhs::sizeRows_,
-    sizeCols_  = Lhs::sizeCols_,
+    sizeRows_  = (orient_ == Arrays::point_) ? 1 : UnknownSize,
+    sizeCols_  = (orient_ == Arrays::vector_) ? 1 : UnknownSize,
     storage_   = Lhs::storage_
   };
   typedef typename Lhs::Type Type;
@@ -342,6 +279,64 @@ struct Traits< SubOperator <Lhs> >
 
 } // namespace hidden
 
+/** @ingroup Arrays
+  * @class SubOperator
+  *
+  * @brief Generic expression when the sub-part of an expression is accessed
+  * (specialization for vectors)
+  *
+  * @tparam Lhs the type of the array or expression to which we are
+  * applying the sub operator.
+  *
+  * This class represents an expression where a sub operator is applied to
+  * an array expression. It is the return type of the sub operation.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name SubOperator type explicitly.
+  */
+template< typename Lhs, int Orient_>
+class SubOperator: public ExprBase< SubOperator<Lhs, Orient_> >, public TRef<1>
+{
+  public:
+    typedef ExprBase< SubOperator<Lhs, Orient_> > Base;
+    typedef typename hidden::Traits< SubOperator< Lhs, Orient_> >::Type Type;
+    typedef typename hidden::Traits< SubOperator< Lhs, Orient_> >::ReturnType ReturnType;
+    enum
+    {
+      structure_ = hidden::Traits< SubOperator<Lhs, Arrays::vector_> >::structure_,
+      orient_    = hidden::Traits< SubOperator<Lhs, Arrays::vector_> >::orient_,
+      sizeRows_  = hidden::Traits< SubOperator<Lhs, Arrays::vector_> >::sizeRows_,
+      sizeCols_  = hidden::Traits< SubOperator<Lhs, Arrays::vector_> >::sizeCols_,
+      storage_   = hidden::Traits< SubOperator<Lhs, Arrays::vector_> >::storage_
+    };
+    /** Type of the Range for the rows */
+    typedef TRange<UnknownSize> RowRange;
+    /** Type of the Range for the columns */
+    typedef TRange<UnknownSize> ColRange;
+
+    /** Constructor */
+    inline SubOperator( Lhs const& lhs, Range const& I, Range const& J)
+                      : Base(), lhs_(lhs), rows_(I), cols_(J) {}
+    /**  @return the range of the rows */
+    inline RowRange const& rowsImpl() const { return rows_;}
+    /** @return the range of the Columns */
+    inline ColRange const&colsImpl() const { return cols_;}
+
+    /** @return the left hand side expression */
+    inline Lhs const& lhs() const { return lhs_; }
+    /** @return the element ith element
+     *  @param i index of the ith element
+     **/
+    inline ReturnType elt1Impl(int i) const { return (lhs_.elt1Impl(i));}
+    /** accesses to the element */
+    inline ReturnType elt0Impl() const { return (lhs_.elt());}
+
+  protected:
+    Lhs const& lhs_;
+    RowRange rows_;
+    ColRange cols_;
+};
+
 /** @ingroup Arrays
   * @class SubOperator
   *
@@ -358,7 +353,8 @@ struct Traits< SubOperator <Lhs> >
   * don't have to name SubOperator type explicitly.
   */
 template< typename Lhs>
-class SubOperator<Lhs, Arrays::vector_ > : public ExprBase< SubOperator<Lhs, Arrays::vector_> >, public TRef<1>
+class SubOperator<Lhs, Arrays::vector_ >: public ExprBase< SubOperator<Lhs, Arrays::vector_> >
+                                        , public TRef<1>
 {
   public:
     typedef ExprBase< SubOperator<Lhs, Arrays::vector_> > Base;
@@ -378,35 +374,20 @@ class SubOperator<Lhs, Arrays::vector_ > : public ExprBase< SubOperator<Lhs, Arr
     typedef TRange<1> ColRange;
 
     /** Constructor */
-    inline SubOperator( Lhs const& lhs, RowRange const& I) : Base(), lhs_(lhs), rows_(I) {}
+    inline SubOperator( Lhs const& lhs, Range const& I) : Base(), lhs_(lhs), rows_(I) {}
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return rows_;}
-    /** @return the first row index of the sub expression */
-    inline int beginRowsImpl() const { return rows_.begin();}
-    /** @return the ending row index of the sub expression */
-    inline int endRowsImpl() const { return rows_.end();}
-    /** @return the number of rows of the sub expression */
-    inline int sizeRowsImpl() const { return rows_.size();}
-
     /** @return the range of the Columns */
     inline ColRange const& colsImpl() const { return lhs_.cols();}
-    /** @return the first column index of the sub expression */
-    inline int beginColsImpl() const { return lhs_.begin();}
-    /** @return the ending column index of the sub expression */
-    inline int endColsImpl() const { return lhs_.end();}
-    /** @return the number of columns of the sub expression */
-    inline int sizeColsImpl() const { return 1;}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
     /** @return the element ith element
      *  @param i index of the ith element
      **/
-    inline ReturnType elt1Impl(int i) const
-    { return (this->asDerived().lhs().elt1Impl(i));}
+    inline ReturnType elt1Impl(int i) const { return (lhs_.elt1Impl(i));}
     /** accesses to the element */
-    inline ReturnType elt0Impl() const
-    { return (this->asDerived().lhs().elt());}
+    inline ReturnType elt0Impl() const { return (lhs_.elt());}
 
   protected:
     Lhs const& lhs_;
@@ -449,24 +430,11 @@ class SubOperator<Lhs, Arrays::point_ > : public ExprBase< SubOperator<Lhs, Arra
     /** Type of the Range for the columns */
     typedef TRange<UnknownSize> ColRange;
     /** Constructor */
-    inline SubOperator( Lhs const& lhs, ColRange const& J) : Base(), lhs_(lhs), cols_(J) {}
+    inline SubOperator( Lhs const& lhs, Range const& J) : Base(), lhs_(lhs), cols_(J) {}
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return lhs_.rows();}
-    /** @return the first row index of the sub expression */
-    inline int beginRowsImpl() const { return lhs_.begin();}
-    /** @return the ending row index of the sub expression */
-    inline int endRowsImpl() const { return lhs_.end();}
-    /** @return the number of rows */
-    inline int sizeRowsImpl() const { return 1;}
-
     /** @return the range of the Columns */
     inline ColRange const& colsImpl() const { return cols_;}
-    /** @return the first column index of the sub expression */
-    inline int beginColsImpl() const { return cols_.begin();}
-    /** @return the ending column index of the sub expression */
-    inline int endColsImpl() const { return cols_.end();}
-    /** @return the number of columns */
-    inline int sizeColsImpl() const { return cols_.size();}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
@@ -484,35 +452,6 @@ class SubOperator<Lhs, Arrays::point_ > : public ExprBase< SubOperator<Lhs, Arra
     ColRange cols_;
 };
 
-/** @ingroup Arrays
-  * @brief implement the access to the elements in the general case.
-  **/
-template< typename Lhs>
-class SubOperatorBase : public ExprBase< SubOperator< Lhs> >
-{
-  public:
-    typedef SubOperator<Lhs> Derived;
-    typedef ExprBase< Derived > Base;
-    typedef typename hidden::Traits< Derived >::Type Type;
-    typedef typename hidden::Traits< Derived >::ReturnType ReturnType;
-    /** constructor. */
-    inline SubOperatorBase() : Base() {}
-    /** @return the element (i,j)
-     *  @param i, j indexes of the row and column
-     **/
-    inline ReturnType elt2Impl(int i, int j) const
-    { return (this->asDerived().lhs().elt(i, j));}
-    /** @return the element ith element
-     *  @param i index of the ith element
-     **/
-    inline ReturnType elt1Impl(int i) const
-    { return (this->asDerived().lhs().elt(i));}
-    /** accesses to the element */
-    inline ReturnType elt0Impl() const
-    { return (this->asDerived().lhs().elt());}
-};
-
-
 } // namespace STK
 
 #endif /* STK_SLICINGOPERATORS_H */
