@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2012  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -36,165 +36,24 @@
 #ifndef STK_ARRAYBYARRAYPRODUCT_H
 #define STK_ARRAYBYARRAYPRODUCT_H
 
-#ifdef _OPENMP
-#include <omp.h>
-#endif
-
 namespace STK
 {
 namespace hidden
 {
-/** @ingroup hidden
- *  This structure encapsulate the data allocated for a panel.
- **/
-template<class Type>
-struct Panel
-{
-  Type panel[blockSize*panelSize];
-  inline Type const& operator[](int i) const { return panel[i];}
-  inline Type& operator[](int i) { return panel[i];}
-};
+template<typename Lhs, typename Rhs, typename Result, bool Orient_> struct bp;
 
 /** @ingroup hidden
- *  This structure encapsulate the data allocated for a block.
- **/
-template<class Type>
-struct Block
-{
-  Type block[blockSize*blockSize];
-  inline Type const& operator[](int i) const { return block[i];}
-  inline Type& operator[](int i) { return block[i];}
-};
-
-/** @ingroup hidden
- *  This structure regroup the methods to used after block multiplication in
- *  order to perform the product of the remaining rows and columns.
+ *  Methods to use for C=AB with A divided in blocks and B divided in panels.
+ *  The structure bp contains only static method and typedef and should normally
+ *  not be used directly.
  **/
 template<typename Lhs, typename Rhs, typename Result>
-struct MultCoefImpl
-{
-  typedef typename Result::Type Type;
-  enum
-  {
-    sizeRows_  = Result::sizeRows_,
-    sizeCols_  = Result::sizeCols_,
-    orient_    = Result::orient_,
-    storage_   = Result::storage_
-  };
-  /** dot product. general by general*/
-  static void dot( Lhs const& lhs, Rhs const& rhs, Result& res, int iRow, int jCol)
-  {
-    res.elt(iRow, jCol) = Type(0);
-    Range const dotRange = Range::inf(lhs.rangeColsInRow(iRow), rhs.rangeRowsInCol(jCol));
-    for (int k=dotRange.begin(); k< dotRange.end(); ++k)
-      res.elt(iRow, jCol) += lhs.elt(iRow, k) * rhs.elt(k, jCol);
-  }
-  /** dot product. general by vector */
-  static void dot( Lhs const& lhs, ITContainer<Rhs, Arrays::vector_> const& rhs
-                 , ICAllocator<Result, sizeRows_, sizeCols_>& res, int iRow)
-  {
-    res.elt(iRow) = Type(0);
-    Range const dotRange = Range::inf(lhs.rangeColsInRow(iRow), rhs.range());
-    for (int k=dotRange.begin(); k< dotRange.end(); ++k)
-      res.elt(iRow) += lhs.elt(iRow, k) * rhs.elt(k);
-  }
-  /** dot product. general by vector */
-  static void dot( ITContainer<Lhs, Arrays::point_> const& lhs
-                 , Rhs const& rhs, ICAllocator<Result, sizeRows_, sizeCols_>& res, int jCol)
-  {
-    res.elt(jCol) = Type(0);
-    Range const dotRange = Range::inf(rhs.rangeRowsInCol(jCol), lhs.range());
-    for (int k=dotRange.begin(); k< dotRange.end(); ++k)
-      res.elt(jCol) += lhs.elt(k) * rhs.elt(k, jCol);
-  }
-  /** multiplication with one sized vectors */
-  static void mult1( Lhs const& lhs, Rhs const& rhs, Result& res
-                   , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-        res.elt(i,j) += lhs.elt(i, lhsCol) * rhs.elt(rhsRow, j);
-  }
-  /** multiplication with two sized vectors */
-  static void mult2( Lhs const& lhs, Rhs const& rhs, Result& res
-                   , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-        res.elt(i,j) += lhs.elt(i, lhsCol  ) * rhs.elt(rhsRow,   j)
-                      + lhs.elt(i, lhsCol+1) * rhs.elt(rhsRow+1, j);
-  }
-  /** multiplication with three sized vectors */
-  static void mult3( Lhs const& lhs, Rhs const& rhs, Result& res
-                   , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-        res.elt(i,j) += lhs.elt(i, lhsCol  ) * rhs.elt(rhsRow, j)
-                      + lhs.elt(i, lhsCol+1) * rhs.elt(rhsRow+1, j)
-                      + lhs.elt(i, lhsCol+2) * rhs.elt(rhsRow+2, j);
-  }
-  /** multiplication with one sized vectors */
-  static void multVec1( Lhs const& lhs, Rhs const& rhs, Result& res
-                      , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      res.elt(i) += lhs.elt(i, lhsCol) * rhs.elt(rhsRow);
-  }
-  /** multiplication with two sized vectors */
-  static void multVec2( Lhs const& lhs, Rhs const& rhs, Result& res
-                      , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      res.elt(i) += lhs.elt(i, lhsCol) * rhs.elt(rhsRow)
-                  + lhs.elt(i, lhsCol+1) * rhs.elt(rhsRow+1);
-  }
-  /** multiplication with three sized vectors */
-  static void multVec3( Lhs const& lhs, Rhs const& rhs, Result& res
-                      , int lhsCol, int rhsRow)
-  {
-    for (int i=lhs.beginRows(); i< lhs.endRows(); ++i)
-      res.elt(i) += lhs.elt(i, lhsCol) * rhs.elt(rhsRow)
-                  + lhs.elt(i, lhsCol+1) * rhs.elt(rhsRow+1)
-                  + lhs.elt(i, lhsCol+2) * rhs.elt(rhsRow+2);
-  }
-  /** multiplication with one sized vectors */
-  static void multPoint1( Lhs const& lhs, Rhs const& rhs, Result& res
-                        , int lhsCol, int rhsRow)
-  {
-    for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-      res.elt(j) += lhs.elt(lhsCol) * rhs.elt(rhsRow, j);
-  }
-  /** multiplication with two sized vectors */
-  static void multPoint2( Lhs const& lhs, Rhs const& rhs, Result& res
-                        , int lhsCol, int rhsRow)
-  {
-    for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-      res.elt(j) += lhs.elt(lhsCol) * rhs.elt(rhsRow, j)
-                  + lhs.elt(lhsCol+1) * rhs.elt(rhsRow+1, j);
-  }
-  /** multiplication with three sized vectors */
-  static void multPoint3( Lhs const& lhs, Rhs const& rhs, Result& res
-                        , int lhsCol, int rhsRow)
-  {
-    for (int j=rhs.beginCols(); j< rhs.endCols(); ++j)
-      res.elt(j) += lhs.elt(lhsCol) * rhs.elt(rhsRow, j)
-                  + lhs.elt(lhsCol+1) * rhs.elt(rhsRow+1, j)
-                  + lhs.elt(lhsCol+2) * rhs.elt(rhsRow+2, j);
-  }
-};
-
-/** @ingroup hidden
- * Methods to use for C=AB with A divided in blocks and B divided in panels.
- * The structure bp contains only static method and typedef and should normally
- * not be used directly.
- **/
-template<typename Lhs, typename Rhs, typename Result>
-struct bp
+struct bp<Lhs, Rhs, Result, (bool)Arrays::by_row_>
 {
   typedef typename Result::Type Type;
   typedef hidden::MultImpl<Type> Cmult;
   typedef hidden::MultCoefImpl<Lhs, Rhs, Result> MultCoeff;
+
   /** Main method for Matrices multiplication implementation.
    *  @note res have been resized and initialized to zero outside this method.
    **/
@@ -203,77 +62,111 @@ struct bp
 #ifdef STK_DEBUG
      stk_cout << _T("Entering bp::run()\n");
 #endif
+    // check small sizes
+    switch (lhs.sizeRows())
+    {
+      case 0:
+        return; break;
+      case 1:
+        MultCoeff::mult1RowOuterCol(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      case 2:
+        MultCoeff::mult2RowOuterCol(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      case 3:
+        MultCoeff::mult3RowOuterCol(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      default: break;
+    }
+    switch (rhs.sizeCols())
+    {
+      case 0:
+        return; break;
+      case 1:
+        MultCoeff::mult1ColOuterCol(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      case 2:
+        MultCoeff::mult2ColOuterCol(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      case 3:
+        MultCoeff::mult3ColOuterCol(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      default: break;
+    }
     // compute dimensions
     int nbInnerLoop = lhs.sizeCols()/blockSize; // = rhs.sizeRows()/blockSize;
-    int nbBlocks = lhs.sizeRows()/blockSize;
-    int nbPanels = rhs.sizeCols()/panelSize;
+    int nbBlocks    = lhs.sizeRows()/blockSize;
+    int nbPanels    = rhs.sizeCols()/panelSize;
     // remaining sizes in the matrices
     int pSize = rhs.sizeCols() - panelSize*nbPanels;
     int bSize = lhs.sizeRows() - blockSize*nbBlocks;
     int tSize = lhs.sizeCols() - blockSize*nbInnerLoop;
-               // = rhs.sizeRows() -  rhs.sizeRows()/blockSize
+              // = rhs.sizeRows() -  rhs.sizeRows()/blockSize
     int iLastRow = lhs.beginRows() + nbBlocks * blockSize;
     int jLastCol = rhs.beginCols() + nbPanels * panelSize;
     int kLastPos = lhs.beginCols() + blockSize * nbInnerLoop;
-    // create panels and blocks
-    Panel<Type>* tabPanel = new Panel<Type>[nbPanels+1];
-    Block<Type>* tabBlock = new Block<Type>[nbBlocks+1];
-    // start blocks by panel
-    for (int k = 0; k<nbInnerLoop; ++k)
+    if (nbInnerLoop)
     {
-      int kPos = lhs.beginCols() + k * blockSize;
-      for (int i = 0; i<nbBlocks; ++i)
+      // create panels and blocks
+      Panel<Type>* tabPanel = new Panel<Type>[nbPanels+1];
+      Block<Type>* tabBlock = new Block<Type>[nbBlocks+1];
+      // start blocks by panel
+      for (int k = 0; k<nbInnerLoop; ++k)
       {
-        int iRow = lhs.beginRows() + i * blockSize;
-        arrayToBlock( lhs, tabBlock[i], iRow, kPos);
-      }
-      arrayToBlock( lhs, tabBlock[nbBlocks], iLastRow, kPos, bSize);
-      for (int j = 0; j<nbPanels; ++j)
-      {
-        int jCol = rhs.beginCols() + j*panelSize;
-        arrayToPanel( rhs, tabPanel[j], kPos, jCol);
-      }
-      arrayToPanel( rhs, tabPanel[nbPanels], kPos, jLastCol, pSize);
+        int kPos = lhs.beginCols() + k * blockSize;
+        for (int i = 0; i<nbBlocks; ++i)
+        {
+          int iRow = lhs.beginRows() + i * blockSize;
+          arrayToBlock( lhs, tabBlock[i], iRow, kPos);
+        }
+        arrayToBlock( lhs, tabBlock[nbBlocks], iLastRow, kPos, bSize);
+        for (int j = 0; j<nbPanels; ++j)
+        {
+          int jCol = rhs.beginCols() + j*panelSize;
+          arrayToPanel( rhs, tabPanel[j], kPos, jCol);
+        }
+        arrayToPanel( rhs, tabPanel[nbPanels], kPos, jLastCol, pSize);
+#ifdef _OPENMP
+#pragma omp parallel for
+#endif
+        for (int i = 0; i<nbBlocks; ++i)
+        {
+          int iRow = lhs.beginRows() + i * blockSize;
+          for (int j = 0; j<nbPanels; ++j)
+          {
+            int jCol = rhs.beginCols() + j * panelSize;
+            blockByPanel( tabBlock[i], tabPanel[j], res, iRow, jCol);
+          }
+        }
+        for (int i = 0; i<nbBlocks; ++i)
+        {
+          int iRow = lhs.beginRows() + i * blockSize;
+          blockByPanel( tabBlock[i], tabPanel[nbPanels], res, iRow, jLastCol, pSize);
+        }
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
-      for (int i = 0; i<nbBlocks; ++i)
-      {
-        int iRow = lhs.beginRows() + i * blockSize;
         for (int j = 0; j<nbPanels; ++j)
         {
           int jCol = rhs.beginCols() + j * panelSize;
-          blockByPanel( tabBlock[i], tabPanel[j], res, iRow, jCol);
+          blockByPanel( tabBlock[nbBlocks], tabPanel[j], res, iLastRow, jCol, panelSize, bSize);
         }
-      }
-      for (int i = 0; i<nbBlocks; ++i)
-      {
-        int iRow = lhs.beginRows() + i * blockSize;
-        blockByPanel( tabBlock[i], tabPanel[nbPanels], res, iRow, jLastCol, pSize);
-      }
-#ifdef _OPENMP
-#pragma omp parallel for
-#endif
-      for (int j = 0; j<nbPanels; ++j)
-      {
-        int jCol = rhs.beginCols() + j * panelSize;
-        blockByPanel( tabBlock[nbBlocks], tabPanel[j], res, iLastRow, jCol, panelSize, bSize);
-      }
-      blockByPanel( tabBlock[nbBlocks], tabPanel[nbPanels], res, iLastRow, jLastCol, pSize, bSize);
-    }
-    delete[] tabBlock;
-    delete[] tabPanel;
+        blockByPanel( tabBlock[nbBlocks], tabPanel[nbPanels], res, iLastRow, jLastCol, pSize, bSize);
+      } // InnerLoop
+      delete[] tabBlock;
+      delete[] tabPanel;
+    } // if IneerLoop
     // treat the remaining rows, columns
     switch (tSize)
     {
       case 1:
-        MultCoeff::mult1(lhs, rhs, res, kLastPos, kLastPos);
+        MultCoeff::mult1Col(lhs, rhs, res, kLastPos, kLastPos);
         break;
       case 2:
-        MultCoeff::mult2(lhs, rhs, res, kLastPos, kLastPos);
+        MultCoeff::mult2Col(lhs, rhs, res, kLastPos, kLastPos);
         break;
       case 3:
-        MultCoeff::mult3(lhs, rhs, res, kLastPos, kLastPos);
+        MultCoeff::mult3Col(lhs, rhs, res, kLastPos, kLastPos);
         break;
       default:
         break;
@@ -311,7 +204,6 @@ struct bp
     }
   }
   /** Default dimension */
-  /** Default dimension */
   static void arrayToPanel( Rhs const& rhs, Panel<Type>& panel, int iRow, int jCol)
   {
     for (int j=0; j<panelSize; ++j)
@@ -400,7 +292,7 @@ struct bp
  * not be used directly.
  **/
 template<typename Lhs, typename Rhs, typename Result>
-struct pb
+struct bp<Lhs, Rhs, Result, (bool)Arrays::by_col_>
 {
   typedef typename Result::Type Type;
   typedef hidden::MultImpl<Type> Cmult;
@@ -410,6 +302,37 @@ struct pb
    **/
   static void run(Lhs const& lhs, Rhs const& rhs, Result& res)
   {
+    // check small sizes
+    switch (lhs.sizeRows())
+    {
+      case 0:
+        return; break;
+      case 1:
+        MultCoeff::mult1RowOuterRow(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      case 2:
+        MultCoeff::mult2RowOuterRow(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      case 3:
+        MultCoeff::mult3RowOuterRow(lhs, rhs, res, lhs.beginRows());
+        return; break;
+      default: break;
+    }
+    switch (rhs.sizeCols())
+    {
+      case 0:
+        return; break;
+      case 1:
+        MultCoeff::mult1ColOuterRow(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      case 2:
+        MultCoeff::mult2ColOuterRow(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      case 3:
+        MultCoeff::mult3ColOuterRow(lhs, rhs, res, rhs.beginCols());
+        return; break;
+      default: break;
+    }
     // compute dimensions
     int nbInnerLoop = lhs.sizeCols()/blockSize; // = rhs.sizeRows()/blockSize;
     int nbBlocks = rhs.sizeCols()/blockSize;
@@ -424,62 +347,65 @@ struct pb
     int lastRow = lhs.beginRows() + panelSize * nbPanels;
     int lastPos = rhs.beginRows() + blockSize * nbInnerLoop;
 
-    // create panels
-    Panel<Type>* tabPanel = new Panel<Type>[nbPanels+1];
-    Block<Type>* tabBlock = new Block<Type>[nbBlocks+1];
-    // start blocks by panel
-    for (int k = 0; k<nbInnerLoop; ++k)
+    if (nbInnerLoop)
     {
-      int kPos = rhs.beginRows() + k * blockSize;
-      // get panels
-      for (int i = 0, iRow= lhs.beginRows(); i<nbPanels; ++i, iRow+= panelSize)
-      { arrayToPanel( lhs, tabPanel[i], iRow, kPos);}
-      arrayToPanel( lhs, tabPanel[nbPanels], lastRow, kPos, pSize);
-      // get blocks
-      for (int j = 0, jCol = rhs.beginCols(); j<nbBlocks; ++j, jCol+=blockSize)
-      { arrayToBlock( rhs, tabBlock[j], kPos, jCol);}
-      arrayToBlock( rhs, tabBlock[nbBlocks], kPos, lastCol, bSize);
-      // perform the products blocks * panel
+      // create panels
+      Panel<Type>* tabPanel = new Panel<Type>[nbPanels+1];
+      Block<Type>* tabBlock = new Block<Type>[nbBlocks+1];
+      // start blocks by panel
+      for (int k = 0; k<nbInnerLoop; ++k)
+      {
+        int kPos = rhs.beginRows() + k * blockSize;
+        // get panels
+        for (int i = 0, iRow= lhs.beginRows(); i<nbPanels; ++i, iRow+= panelSize)
+        { arrayToPanel( lhs, tabPanel[i], iRow, kPos);}
+        arrayToPanel( lhs, tabPanel[nbPanels], lastRow, kPos, pSize);
+        // get blocks
+        for (int j = 0, jCol = rhs.beginCols(); j<nbBlocks; ++j, jCol+=blockSize)
+        { arrayToBlock( rhs, tabBlock[j], kPos, jCol);}
+        arrayToBlock( rhs, tabBlock[nbBlocks], kPos, lastCol, bSize);
+        // perform the products blocks * panel
+#ifdef _OPENMP
+#pragma omp parallel for
+#endif
+        for (int j = 0; j<nbBlocks; ++j)
+        {
+          int jCol = rhs.beginCols() + j * blockSize;
+          for (int i = 0; i<nbPanels; ++i)
+          {
+            int iRow = lhs.beginRows() + i * panelSize;
+            panelByBlock( tabPanel[i], tabBlock[j], res, iRow, jCol);
+          }
+        }
+        for (int j = 0; j<nbBlocks; ++j)
+        {
+          int jCol = rhs.beginCols() + j * blockSize;
+          panelByBlock( tabPanel[nbPanels], tabBlock[j], res, lastRow, jCol, pSize);
+        }
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
-      for (int j = 0; j<nbBlocks; ++j)
-      {
-        int jCol = rhs.beginCols() + j * blockSize;
         for (int i = 0; i<nbPanels; ++i)
         {
-          int iRow = lhs.beginRows() + i * panelSize;
-          panelByBlock( tabPanel[i], tabBlock[j], res, iRow, jCol);
+          int iRow= lhs.beginRows() + i * panelSize;
+          panelByBlock( tabPanel[i],  tabBlock[nbBlocks], res, iRow, lastCol, panelSize, bSize);
         }
-      }
-      for (int j = 0; j<nbBlocks; ++j)
-      {
-        int jCol = rhs.beginCols() + j * blockSize;
-        panelByBlock( tabPanel[nbPanels], tabBlock[j], res, lastRow, jCol, pSize);
-      }
-#ifdef _OPENMP
-#pragma omp parallel for
-#endif
-      for (int i = 0; i<nbPanels; ++i)
-      {
-        int iRow= lhs.beginRows() + i * panelSize;
-        panelByBlock( tabPanel[i],  tabBlock[nbBlocks], res, iRow, lastCol, panelSize, bSize);
-      }
-      panelByBlock( tabPanel[nbPanels],  tabBlock[nbBlocks], res, lastRow, lastCol, pSize, bSize);
-    } // k loop
-    delete[] tabPanel;
-    delete[] tabBlock;
+        panelByBlock( tabPanel[nbPanels],  tabBlock[nbBlocks], res, lastRow, lastCol, pSize, bSize);
+      } // k loop
+      delete[] tabPanel;
+      delete[] tabBlock;
+    }
     // treat the remaining rows, columns
     switch (tSize)
     {
       case 1:
-        MultCoeff::mult1(lhs, rhs, res, lastPos, lastPos);
+        MultCoeff::mult1Col(lhs, rhs, res, lastPos, lastPos);
         break;
       case 2:
-        MultCoeff::mult2(lhs, rhs, res, lastPos, lastPos);
+        MultCoeff::mult2Col(lhs, rhs, res, lastPos, lastPos);
         break;
       case 3:
-        MultCoeff::mult3(lhs, rhs, res, lastPos, lastPos);
+        MultCoeff::mult3Col(lhs, rhs, res, lastPos, lastPos);
         break;
       default:
         break;
