@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2013  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -37,8 +37,7 @@
 
 
 #include "STK_GammaBase.h"
-
-#include "../../../STatistiK/include/STK_Law_Exponential.h"
+#include <STatistiK/include/STK_Law_Exponential.h>
 
 #define MAXITER 400
 #define TOL 1e-8
@@ -52,17 +51,93 @@ namespace Clust
 /** @ingroup Clustering
  * Traits class for the Gamma_ajk_bk traits policy
  **/
-template<class _Array>
-struct MixtureTraits< Gamma_ajk_bk<_Array> >
+template<class Array_>
+struct MixtureTraits< Gamma_ajk_bk<Array_> >
 {
-  typedef _Array Array;
-  typedef typename Array::Type Type;
-  typedef Gamma_ajk_bk_Parameters  Parameters;
-  typedef Array2D<Real>            Param;
+  typedef Array_ Array;
+  typedef ParametersHandler<Clust::Gamma_ajk_bk_> ParamHandler;
 };
 
 } // namespace Clust
 
+/** Specialization of the ParametersHandler struct for Gamma_ajk_bk model */
+template <>
+struct ParametersHandler<Clust::Gamma_ajk_bk_>: public ParametersHandlerGammaBase<  ParametersHandler<Clust::Gamma_ajk_bk_> >
+{
+  typedef ParametersHandlerGammaBase Base;
+  /** shape parameters and statistics */
+  MixtureParametersSet<PointX> shape_;
+  /** scale parameters and statistics */
+  MixtureParametersSet<Real> scale_;
+  /** @return the shape of the kth cluster and jth variable */
+  inline Real const& shapeImpl(int k, int j) const { return shape_[k][j];}
+  /** @return the scale of the kth cluster and jth variable */
+  inline Real const& scaleImpl(int k, int j) const { return scale_[k];}
+  /** copy operator */
+  inline ParametersHandler& operator=( ParametersHandler const& other)
+  { Base::operator =(other);
+    shape_ = other.shape_; scale_ = other.scale_;
+    return *this;
+  }
+  /** copy operator using an array/expression storing the values */
+  template<class Array>
+  inline ParametersHandler& operator=( ExprBase<Array> const& param)
+  {
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { shape_[k][j] = param(k2, j);}
+      scale_[k] = param.row(k2+1).mean();
+    }
+    return *this;
+  }
+
+  /** default constructor */
+  ParametersHandler( int nbCluster)
+                   : Base(nbCluster),shape_(nbCluster), scale_(nbCluster) {}
+  /** copy constructor */
+  ParametersHandler( ParametersHandler const& model)
+                   : Base(model),shape_(model.shape_), scale_(model.scale_) {}
+  /** Initialize the parameters with an array/expression of value */
+  template<class Array>
+  inline ParametersHandler( int nbCluster, ExprBase<Array> const& param)
+                          : Base(nbCluster), shape_(nbCluster), scale_(nbCluster)
+  {
+    Base::resize(param.cols());
+    shape_.resize(param.cols());
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { shape_[k][j] = param(k2, j);}
+      scale_[k] = param.row(k2+1).mean();
+    }
+  }
+  /** destructor */
+  inline ~ParametersHandler() {}
+  /** Initialize the parameters of the model.
+   *  This function initialize the parameters and the statistics.
+   **/
+  void resize(Range const& range)
+  {
+    Base::resize(range);
+    shape_.resize(range);
+    shape_.initialize(1.);
+    scale_.initialize(1.);
+  }
+  /** Store the intermediate results of the Mixture.
+   *  @param iteration Provides the iteration number beginning after the burn-in period.
+   **/
+  inline void storeIntermediateResults(int iteration)
+  { shape_.storeIntermediateResults(iteration); scale_.storeIntermediateResults(iteration);}
+  /** Release the stored results. This is usually used if the estimation
+   *  process failed.
+   **/
+  inline void releaseIntermediateResults()
+  { shape_.releaseIntermediateResults(); scale_.releaseIntermediateResults();}
+  /** set the parameters stored in stat_proba_ and release stat_proba_. */
+  inline void setParameters() { shape_.setParameters(); scale_.setParameters();}
+};
+
 /** @ingroup Clustering
  *  Gamma_ajk_bk is a mixture model of the following form
  * \f[
@@ -76,29 +151,32 @@ template<class Array>
 class Gamma_ajk_bk : public GammaBase< Gamma_ajk_bk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ajk_bk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_ajk_bk<Array> > Base;
-
-    using Base::p_tik;
-    using Base::components;
+    using Base::p_tik; using Base::param_;
     using Base::p_data;
-    using Base::param;
-
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_ajk_bk( int nbCluster) : Base(nbCluster) {}
+    inline Gamma_ajk_bk( int nbCluster): Base(nbCluster) {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_ajk_bk( Gamma_ajk_bk const& model) : Base(model) {}
+    inline Gamma_ajk_bk( Gamma_ajk_bk const& model): Base(model) {}
     /** destructor */
     inline ~Gamma_ajk_bk() {}
-    /** Initialize the component of the model. */
-    void initializeModelImpl() {}
+    /** @return the value of the probability of the i-th sample in the k-th component.
+     *  @param i,k indexes of the sample and of the component
+     **/
+    inline Real lnComponentProbability(int i, int k) const
+    {
+      Real sum =0.;
+      for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
+      { sum += Law::Gamma::lpdf(p_data()->elt(i,j), param_.shape_[k][j], param_.scale_[k]);}
+      return sum;
+    }
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -121,20 +199,19 @@ void Gamma_ajk_bk<Array>::randomInit()
 {
     // compute moments
     this->moments();
-  for (int k= baseIdx; k < components().end(); ++k)
+  for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
   {
     Real value =0;
     for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
     {
       Real mean = meanjk(j,k), variance = variancejk(j,k);
-      param(k).shape_[j] = Law::Exponential::rand((mean*mean/variance));
+      param_.shape_[k][j] = Law::Exponential::rand((mean*mean/variance));
       value += variance/mean;
     }
-    param(k).scale_ = Law::Exponential::rand(value/(this->nbVariable()));
+    param_.scale_[k] = Law::Exponential::rand(value/(this->nbVariable()));
   }
 #ifdef STK_MIXTURE_VERY_VERBOSE
   stk_cout << _T("Gamma_ajk_bk<Array>::randomInit done\n");
-  this->writeParameters(stk_cout);
 #endif
 }
 
@@ -150,21 +227,21 @@ bool Gamma_ajk_bk<Array>::mStep()
   for(iter = 0; iter<MAXITER; ++iter)
   {
     // compute ajk
-    for (int k= baseIdx; k < components().end(); ++k)
+    for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
     {
       for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
       {
         // moment estimate and oldest value
         Real x0 = meanjk(j,k)*meanjk(j,k)/variancejk(j,k);
-        Real x1 = param(k).shape_[j];
+        Real x1 = param_.shape_[k][j];
         if ((x0 <=0.) || !Arithmetic<Real>::isFinite(x0)) return false;
         // compute shape
-        hidden::invPsi f(param(k).meanLog_[j] - std::log(param(k).scale_));
+        hidden::invPsi f(param_.meanLog_[k][j] - std::log(param_.scale_[k]));
         Real a =  Algo::findZero(f, x0, x1, TOL);
 
         if (!Arithmetic<Real>::isFinite(a))
         {
-           param(k).shape_[j] = x0; // use moment estimate
+           param_.shape_[k][j] = x0; // use moment estimate
 #ifdef STK_MIXTURE_DEBUG
           stk_cout << _T("ML estimation failed in Gamma_ajk_bj::mStep()\n");
           stk_cout << "x0 =" << x0 << _T("\n";);
@@ -173,10 +250,10 @@ bool Gamma_ajk_bk<Array>::mStep()
           stk_cout << "f(x1) =" << f(x1) << _T("\n";);
 #endif
         }
-        else { param(k).shape_[j] = a;}
+        else { param_.shape_[k][j] = a;}
       }
       // compute bk
-      param(k).scale_ = param(k).mean_.sum()/param(k).shape_.sum();
+      param_.scale_[k] = param_.mean_[k].sum()/ param_.shape_[k].sum();
     } // end ajk
     // check convergence
     Real value = this->qValue();
