@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2014  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -37,8 +37,7 @@
 #define STK_GAMMA_AK_B_H
 
 #include "STK_GammaBase.h"
-
-#include "../../../STatistiK/include/STK_Law_Exponential.h"
+#include <STatistiK/include/STK_Law_Exponential.h>
 
 #define MAXITER 400
 #define TOL 1e-8
@@ -51,17 +50,96 @@ namespace Clust
 {
 /** @ingroup Clustering
  *  Traits class for the Gamma_ak_b traits policy. */
-template<class _Array>
-struct MixtureTraits< Gamma_ak_b<_Array> >
+template<class Array_>
+struct MixtureTraits< Gamma_ak_b<Array_> >
 {
-  typedef _Array Array;
-  typedef typename Array::Type Type;
-  typedef Gamma_ak_b_Parameters Parameters;
-  typedef Array2D<Real>        Param;
+  typedef Array_ Array;
+  typedef ParametersHandler<Clust::Gamma_ak_b_> ParamHandler;
 };
 
 } // namespace Clust
 
+/** Specialization of the ParametersHandler struct for Gamma_ak_b model */
+template <>
+struct ParametersHandler<Clust::Gamma_ak_b_>: public ParametersHandlerGammaBase<  ParametersHandler<Clust::Gamma_ak_b_> >
+{
+  typedef ParametersHandlerGammaBase Base;
+ /** shape parameters and statistics */
+  MixtureParametersSet<Real> shape_;
+  /** scale parameters and statistics */
+  MixtureParameters<Real> scale_;
+  /** @return the shape of the kth cluster and jth variable */
+  inline Real const& shapeImpl(int k, int j) const { return shape_[k];}
+  /** @return the scale of the kth cluster and jth variable */
+  inline Real const& scaleImpl(int k, int j) const { return scale_();}
+  /** copy operator */
+  inline ParametersHandler& operator=( ParametersHandler const& other)
+  { Base::operator =(other);
+    shape_ = other.shape_; scale_ = other.scale_;
+    return *this;
+  }
+  /** copy operator using an array/expression storing the values */
+  template<class Array>
+  inline ParametersHandler& operator=( ExprBase<Array> const& param)
+  {
+    int nbCluster = mean_().size();
+    scale_() = 0.;
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { scale_() += param(k2+1, j);}
+      shape_[k] = param.row(k2).mean();
+    }
+    scale_() /= (nbCluster*param.sizeCols());
+    return *this;
+  }
+
+  /** default constructor */
+  ParametersHandler( int nbCluster)
+                   : Base(nbCluster), shape_(nbCluster), scale_() {}
+  /** copy constructor */
+  ParametersHandler( ParametersHandler const& model)
+                   : Base(model),shape_(model.shape_), scale_(model.scale_) {}
+  /** Initialize the parameters with an array/expression of value */
+  template<class Array>
+  inline ParametersHandler( int nbCluster, ExprBase<Array> const& param)
+                          : Base(nbCluster), shape_(nbCluster), scale_()
+  {
+    Base::resize(param.cols());
+    scale_() = 0.;
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { scale_() += param(k2+1, j);}
+      shape_[k] = param.row(k2).mean();
+    }
+    scale_() /= (nbCluster*param.sizeCols());
+  }
+  /** destructor */
+  inline ~ParametersHandler() {}
+  /** Initialize the parameters of the model.
+   *  This function initialize the parameters and the statistics.
+   **/
+  void resize(Range const& range)
+  {
+    Base::resize(range);
+    shape_.initialize(1.);
+    scale_.initialize(1.);
+  }
+  /** Store the intermediate results of the Mixture.
+   *  @param iteration Provides the iteration number beginning after the burn-in period.
+   **/
+  inline void storeIntermediateResults(int iteration)
+  { shape_.storeIntermediateResults(iteration); scale_.storeIntermediateResults(iteration);}
+  /** Release the stored results. This is usually used if the estimation
+   *  process failed.
+   **/
+  inline void releaseIntermediateResults()
+  { shape_.releaseIntermediateResults(); scale_.releaseIntermediateResults();}
+  /** set the parameters stored in stat_proba_ and release stat_proba_. */
+  inline void setParameters() { shape_.setParameters(); scale_.setParameters();}
+};
+
 /** @ingroup Clustering
  *  Gamma_ak_b is a mixture model of the following form
  * \f[
@@ -75,53 +153,32 @@ template<class Array>
 class Gamma_ak_b : public GammaBase<Gamma_ak_b<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ak_b<Array> >::Parameters Parameters;
     typedef GammaBase<Gamma_ak_b<Array> > Base;
-
-    using Base::p_tik;
-    using Base::components;
+    using Base::p_tik; using Base::param_;
+    using Base::p_nk;
     using Base::p_data;
-    using Base::param;
-
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_ak_b( int nbCluster) : Base(nbCluster), scale_(), stat_scale_() {}
+    inline Gamma_ak_b( int nbCluster): Base(nbCluster) {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_ak_b( Gamma_ak_b const& model)
-                     : Base(model), scale_(model.scale_), stat_scale_(model.stat_scale_) {}
+    inline Gamma_ak_b( Gamma_ak_b const& model): Base(model) {}
     /** destructor */
     inline ~Gamma_ak_b() {}
-    /** Initialize the component of the model.
-     *  In this interface, the scale_ parameter is shared between all the
-     *  components.
+    /** @return the value of the probability of the i-th sample in the k-th component.
+     *  @param i,k indexes of the sample and of the component
      **/
-    void initializeModelImpl()
+    inline Real lnComponentProbability(int i, int k) const
     {
-      scale_ = 1.;
-      for (int k= baseIdx; k < components().end(); ++k)
-      { param(k).p_scale_ = &scale_;}
-    }
-    /** Store the intermediate results of the Mixture.
-     *  @param iteration Provides the iteration number beginning after the burn-in period.
-     **/
-    void storeIntermediateResultsImpl(int iteration)
-    { stat_scale_.update(scale_);}
-    /** Release the stored results. This is usually used if the estimation
-     *  process failed.
-     **/
-    void releaseIntermediateResultsImpl()
-    { stat_scale_.release();}
-    /** set the parameters stored in stat_proba_ and release stat_proba_. */
-    void setParametersImpl()
-    {
-      scale_ = stat_scale_.param_;
-      stat_scale_.release();
+      Real sum =0.;
+      for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
+      { sum += Law::Gamma::lpdf(p_data()->elt(i,j), param_.shape_[k], param_.scale_());}
+      return sum;
     }
     /** Initialize randomly the parameters of the Gaussian mixture. The centers
      *  will be selected randomly among the data set and the standard-deviation
@@ -132,12 +189,6 @@ class Gamma_ak_b : public GammaBase<Gamma_ak_b<Array> >
     bool mStep();
     /** @return the number of free parameters of the model */
     inline int computeNbFreeParameters() const { return this->nbCluster() + 1;}
-
-  protected:
-    /** common scale */
-    Real scale_;
-    /** Array of the statistics */
-    MixtureStatReal stat_scale_;
 };
 
 /* Initialize randomly the parameters of the Gaussian mixture. The centers
@@ -151,17 +202,16 @@ void Gamma_ak_b<Array>::randomInit()
     this->moments();
   // simulates ak
   Real value = 0.;
-  for (int k= baseIdx; k < components().end(); ++k)
+  for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
   {
     Real mean = this->meank(k), variance = this->variancek(k);
-    param(k).shape_ = Law::Exponential::rand((mean*mean/variance));
-    value += param(k).tk_ * variance/mean;
+    param_.shape_[k]= Law::Exponential::rand((mean*mean/variance));
+    value += p_nk()->elt(k) * variance/mean;
   }
   // simulate b
-  scale_ = Law::Exponential::rand(value/(this->nbVariable()));
+  param_.scale_() = Law::Exponential::rand(value/(this->nbSample()));
 #ifdef STK_MIXTURE_VERY_VERBOSE
   stk_cout << _T("Gamma_ak_b<Array>::randomInit done\n");
-  this->writeParameters(stk_cout);
 #endif
 }
 
@@ -177,20 +227,20 @@ bool Gamma_ak_b<Array>::mStep()
   {
     // compute ak
     Real num = 0., den = 0.;
-    for (int k= baseIdx; k < components().end(); ++k)
+    for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
     {
       // moment estimate and oldest value
-      Real x0 = (param(k).mean_.square()/param(k).variance_).mean();
-      Real x1 = param(k).shape_;
+      Real x0 = (param_.mean_[k].square()/param_.variance_[k]).mean();
+      Real x1 =  param_.shape_[k];
       if ((x0 <=0.) || !Arithmetic<Real>::isFinite(x0)) return false;
 
       // compute shape
-      hidden::invPsi f((param(k).meanLog_ - std::log(scale_)).mean());
+      hidden::invPsi f((param_.meanLog_[k] - std::log(param_.scale_())).mean());
       Real a =  Algo::findZero(f, x0, x1, TOL);
 
       if (!Arithmetic<Real>::isFinite(a))
       {
-        param(k).shape_ = x0; // use moment estimate
+        param_.shape_[k]= x0; // use moment estimate
 #ifdef STK_MIXTURE_DEBUG
         stk_cout << _T("ML estimation failed in Gamma_ak_bj::mStep()\n");
         stk_cout << "x0 =" << x0 << _T("\n";);
@@ -199,16 +249,16 @@ bool Gamma_ak_b<Array>::mStep()
         stk_cout << "f(x1) =" << f(x1) << _T("\n";);
 #endif
       }
-      else { param(k).shape_ = a;}
+      else { param_.shape_[k]= a;}
       // update num and den
-      num += this->meank(k)     * param(k).tk_;
-      den += param(k).shape_ * param(k).tk_;
+      num += this->meank(k)     * p_nk()->elt(k);
+      den += param_.shape_[k]* p_nk()->elt(k);
     }
     // compute b
     Real b = num/den;
     // divergence
     if (!Arithmetic<Real>::isFinite(b)) { return false;}
-    scale_ = b;
+    param_.scale_() = b;
     // check convergence
     Real value = this->qValue();
 #ifdef STK_MIXTURE_DEBUG
