@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2013  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -36,8 +36,7 @@
 #define STK_GAMMA_AK_BJ_H
 
 #include "STK_GammaBase.h"
-
-#include "../../../STatistiK/include/STK_Law_Exponential.h"
+#include <STatistiK/include/STK_Law_Exponential.h>
 
 #define MAXITER 400
 #define TOL 1e-8
@@ -50,17 +49,100 @@ namespace Clust
 {
 /** @ingroup Clustering
  *  Traits class for the Gamma_ak_bj traits policy. */
-template<class _Array>
-struct MixtureTraits< Gamma_ak_bj<_Array> >
+template<class Array_>
+struct MixtureTraits< Gamma_ak_bj<Array_> >
 {
-  typedef _Array Array;
-  typedef typename Array::Type Type;
-  typedef Gamma_ak_bj_Parameters Parameters;
-  typedef Array2D<Real>        Param;
+  typedef Array_ Array;
+  typedef ParametersHandler<Clust::Gamma_ak_bj_> ParamHandler;
 };
 
 } // namespace Clust
 
+/** Specialization of the ParametersHandler struct for Gamma_ak_bj model */
+template <>
+struct ParametersHandler<Clust::Gamma_ak_bj_>: public ParametersHandlerGammaBase<  ParametersHandler<Clust::Gamma_ak_bj_> >
+{
+  typedef ParametersHandlerGammaBase Base;
+  /** shape parameters and statistics */
+  MixtureParametersSet<Real> shape_;
+  /** scale parameters and statistics */
+  MixtureParameters<PointX> scale_;
+  /** @return the shape of the kth cluster and jth variable */
+  inline Real const& shapeImpl(int k, int j) const { return shape_[k];}
+  /** @return the scale of the kth cluster and jth variable */
+  inline Real const& scaleImpl(int k, int j) const { return scale_()[j];}
+  /** copy operator */
+  inline ParametersHandler& operator=( ParametersHandler const& other)
+  { Base::operator =(other);
+    shape_ = other.shape_; scale_ = other.scale_;
+    return *this;
+  }
+  /** copy operator using an array/expression storing the values */
+  template<class Array>
+  inline ParametersHandler& operator=( ExprBase<Array> const& param)
+  {
+    int nbCluster = mean_().size();
+    for (int j= param.beginCols();  j< param.endCols(); ++j)
+    {
+      scale_()[j] = 0.;
+      for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+      { scale_()[j] += param(k2+1, j);}
+      scale_()[j] /= nbCluster;
+    }
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    { shape_[k] = param.row(k2).mean();}
+    return *this;
+  }
+
+  /** default constructor */
+  ParametersHandler( int nbCluster)
+                   : Base(nbCluster), shape_(nbCluster), scale_() {}
+  /** copy constructor */
+  ParametersHandler( ParametersHandler const& model)
+                   : Base(model), shape_(model.shape_), scale_(model.scale_) {}
+  /** Initialize the parameters with an array/expression of value */
+  template<class Array>
+  inline ParametersHandler( int nbCluster, ExprBase<Array> const& param)
+                          : Base(nbCluster), shape_(nbCluster), scale_()
+  {
+    Base::resize(param.cols());
+    scale_.resize(param.cols());
+    for (int j= param.beginCols();  j< param.endCols(); ++j)
+    {
+      scale_()[j] = 0.;
+      for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+      { scale_()[j] += param(k2+1, j);}
+      scale_()[j] /= nbCluster;
+    }
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    { shape_[k] = param.row(k2).mean();}
+  }
+  /** destructor */
+  inline ~ParametersHandler() {}
+  /** Initialize the parameters of the model.
+   *  This function initialize the parameters and the statistics.
+   **/
+  void resize(Range const& range)
+  {
+    Base::resize(range);
+    shape_.initialize(1.);
+    scale_.resize(range);
+    scale_.initialize(1.);
+  }
+  /** Store the intermediate results of the Mixture.
+   *  @param iteration Provides the iteration number beginning after the burn-in period.
+   **/
+  inline void storeIntermediateResults(int iteration)
+  { shape_.storeIntermediateResults(iteration); scale_.storeIntermediateResults(iteration);}
+  /** Release the stored results. This is usually used if the estimation
+   *  process failed.
+   **/
+  inline void releaseIntermediateResults()
+  { shape_.releaseIntermediateResults(); scale_.releaseIntermediateResults();}
+  /** set the parameters stored in stat_proba_ and release stat_proba_. */
+  inline void setParameters() { shape_.setParameters(); scale_.setParameters();}
+};
+
 /** @ingroup Clustering
  *  Gamma_ak_bj is a mixture model of the following form
  * \f[
@@ -71,58 +153,35 @@ struct MixtureTraits< Gamma_ak_bj<_Array> >
  * \f]
  **/
 template<class Array>
-class Gamma_ak_bj : public GammaBase<Gamma_ak_bj<Array> >
+class Gamma_ak_bj: public GammaBase< Gamma_ak_bj<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ak_bj<Array> >::Parameters Parameters;
     typedef GammaBase<Gamma_ak_bj<Array> > Base;
-
-    using Base::p_tik;
-    using Base::components;
+    using Base::p_tik; using Base::param_;
+    using Base::p_nk;
     using Base::p_data;
-    using Base::param;
-
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_ak_bj( int nbCluster) : Base(nbCluster), scale_(), stat_scale_() {}
+    inline Gamma_ak_bj( int nbCluster): Base(nbCluster) {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_ak_bj( Gamma_ak_bj const& model)
-                      : Base(model), scale_(model.scale_), stat_scale_(model.stat_scale_) {}
+    inline Gamma_ak_bj( Gamma_ak_bj const& model): Base(model) {}
     /** destructor */
     inline ~Gamma_ak_bj() {}
-    /** Initialize the component of the model.
-     *  In this interface, the scale_ parameter is shared between all the
-     *  components.
+    /** @return the value of the probability of the i-th sample in the k-th component.
+     *  @param i,k indexes of the sample and of the component
      **/
-    void initializeModelImpl()
+    inline Real lnComponentProbability(int i, int k) const
     {
-      scale_.resize(p_data()->cols());
-      scale_ = 1.;
-      for (int k= baseIdx; k < components().end(); ++k)
-      { param(k).p_scale_ = &scale_;}
-      stat_scale_.initialize(p_data()->cols());
-    }
-    /** Store the intermediate results of the Mixture.
-     *  @param iteration Provides the iteration number beginning after the burn-in period.
-     **/
-    void storeIntermediateResultsImpl(int iteration)
-    { stat_scale_.update(scale_);}
-    /** Release the stored results. This is usually used if the estimation
-     *  process failed.
-     **/
-    void releaseIntermediateResultsImpl()
-    { stat_scale_.release();}
-    /** set the parameters stored in stat_proba_ and release stat_proba_. */
-    void setParametersImpl()
-    {
-      scale_ = stat_scale_.param_;
-      stat_scale_.release();
+      Real sum =0.;
+      for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
+      { sum += Law::Gamma::lpdf(p_data()->elt(i,j), param_.shape_[k], param_.scale_()[j]);}
+      return sum;
     }
     /** Initialize randomly the parameters of the Gaussian mixture. The centers
      *  will be selected randomly among the data set and the standard-deviation
@@ -134,12 +193,6 @@ class Gamma_ak_bj : public GammaBase<Gamma_ak_bj<Array> >
     /** @return the number of free parameters of the model */
     inline int computeNbFreeParameters() const
     { return this->nbCluster()+ this->nbVariable();}
-
-  protected:
-    /** Array of the common scale */
-    Array2DPoint<Real> scale_;
-    /** Vector of the statistics */
-    MixtureStatVector stat_scale_;
 };
 
 /* Initialize randomly the parameters of the Gaussian mixture. The centers
@@ -152,7 +205,7 @@ void Gamma_ak_bj<Array>::randomInit()
     // compute moments
     this->moments();
   // simulates ak
-  for (int k= baseIdx; k < components().end(); ++k)
+  for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
   {
     Real value= 0.;
     for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
@@ -160,22 +213,21 @@ void Gamma_ak_bj<Array>::randomInit()
       Real mean = meanjk(j,k), variance = variancejk(j,k);
       value += mean*mean/variance;
     }
-    param(k).shape_ = Law::Exponential::rand(value/(this->nbVariable()));
+    param_.shape_[k]= Law::Exponential::rand(value/(this->nbVariable()));
   }
   // simulate bj
   for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
   {
     Real value= 0.;
-    for (int k= baseIdx; k < components().end(); ++k)
+    for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
     {
       Real mean = meanjk(j,k), variance = variancejk(j,k);
-      value += param(k).tk_ * variance/mean;
+      value += p_nk()->elt(k) * variance/mean;
     }
-    scale_[j] = Law::Exponential::rand(value/(this->nbSample()));
+    param_.scale_()[j] = Law::Exponential::rand(value/(this->nbSample()));
   }
 #ifdef STK_MIXTURE_VERY_VERBOSE
   stk_cout << _T("Gamma_ak_bj<Array>::randomInit done\n");
-  this->writeParameters(stk_cout);
 #endif
 }
 
@@ -190,20 +242,20 @@ bool Gamma_ak_bj<Array>::mStep()
   for(iter=0; iter<MAXITER; ++iter)
   {
     // compute ak
-    for (int k= baseIdx; k < components().end(); ++k)
+    for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
     {
       // moment estimate and oldest value
-      Real x0 = (param(k).mean_.square()/param(k).variance_).mean();
-      Real x1 = param(k).shape_;
+      Real x0 = (param_.mean_[k].square()/param_.variance_[k]).mean();
+      Real x1 =  param_.shape_[k];
       if ((x0 <=0.) || !Arithmetic<Real>::isFinite(x0)) return false;
 
       // compute shape
-      hidden::invPsi f((param(k).meanLog_ - scale_.log()).mean());
+      hidden::invPsi f((param_.meanLog_[k] - param_.scale_().log()).mean());
       Real a =  Algo::findZero(f, x0, x1, TOL);
 
       if (!Arithmetic<Real>::isFinite(a))
       {
-        param(k).shape_ = x0; // use moment estimate
+        param_.shape_[k]= x0; // use moment estimate
 #ifdef STK_MIXTURE_DEBUG
         stk_cout << _T("ML estimation failed in Gamma_ak_bj::mStep()\n");
         stk_cout << "x0 =" << x0 << _T("\n";);
@@ -212,22 +264,22 @@ bool Gamma_ak_bj<Array>::mStep()
         stk_cout << "f(x1) =" << f(x1) << _T("\n";);
 #endif
       }
-      else { param(k).shape_ = a;}
+      else { param_.shape_[k]= a;}
     }
     // update all the b^j
     for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
     {
       Real num = 0., den = 0.;
-      for (int k= baseIdx; k < components().end(); ++k)
+      for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
       {
-        num += param(k).mean_[j] * param(k).tk_;
-        den += param(k).shape_   * param(k).tk_;
+        num += param_.mean_[k][j] * p_nk()->elt(k);
+        den += param_.shape_[k]  * p_nk()->elt(k);
       }
       // compute b_j
       Real b = num/den;
       // divergence
       if (!Arithmetic<Real>::isFinite(b)) { return false;}
-      scale_[j] = b;
+      param_.scale_()[j] = b;
     }
     // check convergence
     Real value = this->qValue();
