@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2013  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -36,8 +36,7 @@
 #define STK_GAMMA_AK_BJK_H
 
 #include "STK_GammaBase.h"
-
-#include "../../../STatistiK/include/STK_Law_Exponential.h"
+#include <STatistiK/include/STK_Law_Exponential.h>
 
 namespace STK
 {
@@ -52,13 +51,90 @@ template<class _Array>
 struct MixtureTraits< Gamma_ak_bjk<_Array> >
 {
   typedef _Array Array;
-  typedef typename Array::Type Type;
-  typedef Gamma_ak_bjk_Parameters        Parameters;
-  typedef Array2D<Real>        Param;
+
+  typedef ParametersHandler<Clust::Gamma_ak_bjk_> ParamHandler;
 };
 
 } // namespace Clust
 
+/** Specialization of the ParametersHandler struct for Gamma_ak_bjk model */
+template <>
+struct ParametersHandler<Clust::Gamma_ak_bjk_>: public ParametersHandlerGammaBase<  ParametersHandler<Clust::Gamma_ak_bjk_> >
+{
+    typedef ParametersHandlerGammaBase Base;
+  /** shape parameters and statistics */
+  MixtureParametersSet<Real> shape_;
+  /** scale parameters and statistics */
+  MixtureParametersSet<PointX> scale_;
+  /** @return the shape of the kth cluster and jth variable */
+  inline Real const& shapeImpl(int k, int j) const { return shape_[k];}
+  /** @return the scale of the kth cluster and jth variable */
+  inline Real const& scaleImpl(int k, int j) const { return scale_[k][j];}
+  /** copy operator */
+  inline ParametersHandler& operator=( ParametersHandler const& other)
+  { Base::operator =(other);
+    shape_ = other.shape_; scale_ = other.scale_;
+    return *this;
+  }
+  /** copy operator using an array/expression storing the values */
+  template<class Array>
+  inline ParametersHandler& operator=( ExprBase<Array> const& param)
+  {
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { scale_[k][j] = param(k2+1, j);}
+      shape_[k] = param.row(k2).mean();
+    }
+    return *this;
+  }
+
+  /** default constructor */
+  ParametersHandler( int nbCluster)
+                   : Base(nbCluster), shape_(nbCluster), scale_(nbCluster) {}
+  /** copy constructor */
+  ParametersHandler( ParametersHandler const& model)
+                   : Base(model), shape_(model.shape_), scale_(model.scale_) {}
+  /** Initialize the parameters with an array/expression of value */
+  template<class Array>
+  inline ParametersHandler( int nbCluster, ExprBase<Array> const& param)
+                          : Base(nbCluster), shape_(nbCluster), scale_(nbCluster)
+  {
+    Base::resize(param.cols());
+    scale_.resize(param.cols());
+    for (int k2= param.beginRows(), k= param.beginRows(); k2 < param.endRows(); k2+=2, k++)
+    {
+      for (int j= param.beginCols();  j< param.endCols(); ++j)
+      { scale_[k][j] = param(k2+1, j);}
+      shape_[k] = param.row(k2).mean();
+    }
+  }
+  /** destructor */
+  inline ~ParametersHandler() {}
+  /** Initialize the parameters of the model.
+   *  This function initialize the parameters and the statistics.
+   **/
+  void resize(Range const& range)
+  {
+    Base::resize(range);
+    shape_.initialize(1.);
+    scale_.resize(range);
+    scale_.initialize(1.);
+  }
+  /** Store the intermediate results of the Mixture.
+   *  @param iteration Provides the iteration number beginning after the burn-in period.
+   **/
+  inline void storeIntermediateResults(int iteration)
+  { shape_.storeIntermediateResults(iteration); scale_.storeIntermediateResults(iteration);}
+  /** Release the stored results. This is usually used if the estimation
+   *  process failed.
+   **/
+  inline void releaseIntermediateResults()
+  { shape_.releaseIntermediateResults(); scale_.releaseIntermediateResults();}
+  /** set the parameters stored in stat_proba_ and release stat_proba_. */
+  inline void setParameters() { shape_.setParameters(); scale_.setParameters();}
+};
+
 /** @ingroup Clustering
  *  Gamma_ak_bjk is a mixture model of the following form
  * \f[
@@ -72,27 +148,32 @@ template<class Array>
 class Gamma_ak_bjk : public GammaBase< Gamma_ak_bjk<Array> >
 {
   public:
-    typedef typename Clust::MixtureTraits< Gamma_ak_bjk<Array> >::Parameters Parameters;
     typedef GammaBase< Gamma_ak_bjk<Array> > Base;
-
-    using Base::p_tik;
-    using Base::components;
+    using Base::p_tik; using Base::param_;
     using Base::p_data;
-    using Base::param;
-
     using Base::meanjk;
     using Base::variancejk;
 
     /** default constructor
      * @param nbCluster number of cluster in the model
      **/
-    inline Gamma_ak_bjk( int nbCluster) : Base(nbCluster) {}
+    inline Gamma_ak_bjk( int nbCluster): Base(nbCluster) {}
     /** copy constructor
      *  @param model The model to copy
      **/
-    inline Gamma_ak_bjk( Gamma_ak_bjk const& model) : Base(model) {}
+    inline Gamma_ak_bjk( Gamma_ak_bjk const& model): Base(model) {}
     /** destructor */
     inline ~Gamma_ak_bjk() {}
+    /** @return the value of the probability of the i-th sample in the k-th component.
+     *  @param i,k indexes of the sample and of the component
+     **/
+    inline Real lnComponentProbability(int i, int k) const
+    {
+      Real sum =0.;
+      for (int j=p_data()->beginCols(); j<p_data()->endCols(); ++j)
+      { sum += Law::Gamma::lpdf(p_data()->elt(i,j), param_.shape_[k], param_.scale_[k][j]);}
+      return sum;
+    }
     /** Initialize randomly the parameters of the Gamma mixture. The shape
      *  will be selected randomly using an exponential of parameter mean^2/variance
      *  and the scale will be selected randomly using an exponential of parameter
@@ -115,20 +196,19 @@ void Gamma_ak_bjk<Array>::randomInit()
 {
     // compute moments
     this->moments();
-  for (int k= baseIdx; k < components().end(); ++k)
+  for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
   {
     Real value = 0.0;
     for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
     {
       Real mean = meanjk(j,k), variance = variancejk(j,k);
-      param(k).scale_[j] = Law::Exponential::rand((variance/mean));
+      param_.scale_[k][j] = Law::Exponential::rand((variance/mean));
       value += mean*mean/variance;
     }
-    param(k).shape_ = Law::Exponential::rand(value/(this->nbVariable()));
+    param_.shape_[k]= Law::Exponential::rand(value/(this->nbVariable()));
   }
 #ifdef STK_MIXTURE_VERY_VERBOSE
   stk_cout << _T("Gamma_ak_bjk<Array>::randomInit done\n");
-  this->writeParameters(stk_cout);
 #endif
 }
 
@@ -138,15 +218,15 @@ bool Gamma_ak_bjk<Array>::mStep()
 {
   if (!this->moments()) { return false;}
   // estimate a and b
-  for (int k= baseIdx; k < components().end(); ++k)
+  for (int k= p_tik()->beginCols(); k < p_tik()->endCols(); ++k)
   {
     // moment estimate and oldest value
-    Real x0 = (param(k).mean_.square()/param(k).variance_).mean();
-    Real x1 = param(k).shape_;
+    Real x0 = (param_.mean_[k].square()/param_.variance_[k]).mean();
+    Real x1 =  param_.shape_[k];
     if ((x0 <=0.) || (isNA(x0))) return false;
 
     // get shape
-    hidden::invPsiMLog f( (param(k).meanLog_-param(k).mean_.log()).mean() );
+    hidden::invPsiMLog f( (param_.meanLog_[k]-param_.mean_[k].log()).mean() );
     Real a = Algo::findZero(f, x0, x1, 1e-08);
     if (!Arithmetic<Real>::isFinite(a))
     {
@@ -160,9 +240,9 @@ bool Gamma_ak_bjk<Array>::mStep()
       a = x0; // use moment estimate
     }
     // set values
-    param(k).shape_ = a;
+    param_.shape_[k]= a;
     for (int j=p_data()->beginCols(); j < p_data()->endCols(); ++j)
-    { param(k).scale_[j] = param(k).mean_[j]/a; }
+    { param_.scale_[k][j] = param_.mean_[k][j]/a; }
   }
   return true;
 }
