@@ -49,7 +49,7 @@ namespace STK
 /** @ingroup Base
   *  @brief STK fundamental type of discrete values.
   *
-  *  The type Interger is defined for the numerical computation and the
+  *  The type Integer is defined for the numerical computation and the
   *  internal representation of the discrete variables.
   **/
 typedef int Integer ;
@@ -78,8 +78,8 @@ struct Arithmetic<Integer>  : public std::numeric_limits<Integer>
    **/
   static inline bool isNA(Integer const& x) throw()
   { return (x==std::numeric_limits<Integer>::min());}
-  /** We are using the maximal value (positive or negative) of the Integer
-    * type for NA values. */
+  /** We are using the minimal value of the Integer type for NA values, thus
+   *  we add one in order to compute the available minimal value. */
   static inline Integer min() throw() { return std::numeric_limits<Integer>::min() +1; }
   /** @return @c true if x is  infinite : always false for Integer.
    *  @param x the Integer number to test.
@@ -156,8 +156,12 @@ struct IdTypeImpl<const Integer>
  *  does not match any known name, the NA value is returned.
  **/
 inline Integer stringToInt( String const& str)
-{ return stringToType<Integer>(str);}
-
+{
+  Integer x;
+  istringstream is(str);
+  is >> Proxy<Integer>(x);
+  return x;
+}
 /** @ingroup Base
  *  Convert a String to an Integer using a map.
  *  @param str the String we want to convert
@@ -165,7 +169,11 @@ inline Integer stringToInt( String const& str)
  *  @return the Int represented by the String @c type. if the string
  *  does not match any known name, the @c unknown_ type is returned.
  **/
-Integer stringToInt( String const& str, std::map<String, Integer> const& mapping);
+inline Integer stringToInt( String const& str, std::map<String, Integer> const& mapping)
+{
+  std::map<String, Integer>::const_iterator it=mapping.find(str);
+  return (it == mapping.end()) ? Arithmetic<Integer>::NA() : it->second;
+}
 
 /** @ingroup Base
  *  Convert an Integer to a String.
@@ -174,7 +182,12 @@ Integer stringToInt( String const& str, std::map<String, Integer> const& mapping
  *  @return the string associated to this value.
  **/
 inline String intToString( Integer const& value, std::ios_base& (*f)(std::ios_base&) = std::dec)
-{ return typeToString<Integer>(value,f);}
+{
+  if (Arithmetic<Integer>::isNA(value)) return stringNa;
+  ostringstream os;
+  os << f << value;
+  return os.str();
+}
 
 /** @ingroup Base
  *  Convert an Integer to a String.
@@ -182,7 +195,29 @@ inline String intToString( Integer const& value, std::ios_base& (*f)(std::ios_ba
  *  @param mapping the mapping between Integer and String
  *  @return the String associated to this value.
  **/
-String intToString( Integer const& value, std::map<Integer, String> const& mapping);
+inline String intToString( Integer const& value, std::map<Integer, String> const& mapping)
+{
+  std::map<Integer, String>::const_iterator it=mapping.find(value);
+  return (it == mapping.end()) ? stringNa : it->second;
+}
+
+/** @ingroup Base
+ *  @brief specialization for Integer
+ *  @param s the String to convert
+ *  @return The value to get from the String
+ **/
+template<>
+inline Integer stringToType<Integer>( String const& s)
+{ return stringToInt(s);}
+
+/** @ingroup Base
+ *  @brief specialization for Integer
+ *  @param t The Int to convert to String
+ *  @param f format, by default write every number in decimal
+ **/
+template<>
+inline String typeToString<Integer>( Integer const& t, std::ios_base& (*f)(std::ios_base&))
+{ return intToString(t, f);}
 
 } // namespace STK
 
