@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2009  Serge Iovleff
+/*     Copyright (C) 2004-2015  Serge Iovleff
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -36,7 +36,9 @@
 #ifndef STK_RANGE_H
 #define STK_RANGE_H
 
-#include "STK_Constants.h"
+#include "STK_String.h"
+#include <map>
+
 
 /** Utility macro that can be used in a similar way that first:last */
 #define _R(first,last) Range(first, last, 0)
@@ -54,6 +56,183 @@ typedef TRange<UnknownSize> Range;
 template<int Size_>
 ostream& operator<< (ostream& s, TRange<Size_> const& I);
 
+/** @ingroup STKernel
+ *  @brief Index sub-vector region with fixed size.
+ *
+ *  A Range is an ordered pair [first,last] denoting a sub-vector
+ *  region, similar to a Fortran 90 or Matlab colon notation.
+ *  For example :
+ *  @code
+ *  Vector A(10), B(Range(0,20));
+ *  Range I(2,4, false);
+ *  A(I) = B(Range(0,2, false));
+ *  @endcode
+ *  overwrite the elements (2, 3, 4) of A by the elements (0, 1, 2) of B.
+ *  There is no stride argument, only contiguous regions are allowed.
+ **/
+template<int Size_>
+class TRange
+{
+  public:
+    /** Default constructor. Null range.*/
+    inline TRange() : begin_(baseIdx) {}
+    /** constructor. The first index is defined by the baseIdx macro.
+     **/
+    inline TRange( int) : begin_(baseIdx) {}
+    /** Complete constructor. Give the beginning and the size of the sub-region.
+     *  @param first is the beginning of the sub-region
+     **/
+    inline TRange( int first, int) : begin_(first) {}
+    /** Complete constructor. Give the beginning and the end of the sub-region.
+     * @param first is the beginning of the sub-region
+     **/
+    inline TRange( int first, int, bool) : begin_(first) {}
+    /** Create a copy of an existing TRange.
+     *  @param I The index to copy
+     **/
+    template<int OtherSize_>
+    inline TRange(TRange<OtherSize_> const& I) : begin_(I.begin()) {}
+    /** destructor. */
+    inline ~TRange() {}
+    /** get the first index of the TRange.
+     *  @return the first index of the range
+     **/
+    inline int begin()  const { return begin_;};
+    /** get the ending index of the TRange.
+     *  @return the first index of the range
+     **/
+    inline int end()  const { return begin_ + Size_;};
+    /** get the size of the TRange (the number of elements).
+     *  @return the size of the range
+     **/
+    inline int size()   const { return Size_;};
+    // backward compatibility
+    /** get the first index of the TRange.
+     *  @return the first index of the range
+     **/
+    inline int firstIdx()  const { return begin_;};
+    /** get the last index of the TRange.
+     *  @return the last index of the range
+     **/
+    inline int lastIdx() const { return begin_ + Size_ -1;};
+    /** check if the range is empty or not.
+     *  @return @c true if size <=0, @c false otherwise
+     */
+    inline bool empty() const { return Size_<=0;};
+
+    /** Shift the TRange giving the first element.
+     *  @param first new value of the first element. */
+    inline TRange& shift(int first) { return inc(first - begin_);}
+    /** create the TRange [begin_+inc, end_+inc_].
+     *  @param inc the increment to apply
+     **/
+    inline TRange& inc(int inc) { begin_ +=inc; return *this;}
+    /** create the TRange [begin_+inc, end_].
+     *  @param inc the increment to apply
+     **/
+    inline TRange& incFirst(int inc) { begin_ +=inc; return *this;}
+    /** create the TRange [begin_, end_+inc]
+     * @param inc the increment to apply
+     **/
+    inline TRange& incLast(int inc) { begin_ +=inc; return *this;}
+    /** create the TRange [begin_-dec, end_-dec]
+     *  @param dec the decrement to apply
+     **/
+    inline TRange& dec(int dec) { begin_ -=dec; return *this;}
+    /** @brief create the TRange [begin_-dec, end_]
+     * @param dec the decrement to apply
+     **/
+    inline TRange& decFirst(int dec) { begin_ -=dec;  return *this;}
+    /** create the TRange [begin_, end_-dec]
+     * @param dec the decrement to apply
+     **/
+    inline TRange& decLast(int dec){ begin_ -=dec; return *this;}
+    /** @return this %TRange incremented **/
+    inline TRange& operator+=(int inc) { begin_ += inc; return *this;}
+    /** @return this %TRange decremented **/
+    inline TRange& operator-=(int dec) { begin_ -= dec; return *this;}
+    /** @return this %TRange incremented **/
+    inline TRange operator+(int inc) const { return(TRange(begin_+inc, Size_));}
+    /** @return %TRange I incremented **/
+    inline friend TRange operator+(int inc, TRange const &I) { return I+inc;}
+    /** @return this %TRange decremented **/
+    inline TRange operator-(int dec) const { return(TRange(begin_-dec, Size_));}
+
+    /** check if this TRange in include in an other TRange
+     *  @param I the index to compare
+     *  @return @c true if this is include in @c I, @c false otherwise
+     **/
+    template<int OtherSize_>
+    inline bool isIn(TRange<OtherSize_> const& I) const
+    { return ((begin_>= I.begin())&&(end()<=I.end()));}
+    /** check if the TRange I is include in the this TRange
+     *  @param I the range to compare
+     *  @return @c true if this contain I, @c false otherwise
+     **/
+    template<int OtherSize_>
+    inline bool isContaining(TRange<OtherSize_> const& I) const
+    { return ((begin_<= I.begin())&&(end()>=I.end()));}
+    /** Return true if i is in this TRange
+     *  @param i the integer to compare
+     *  @return @c true if i is in this, @c false otherwise
+     **/
+    inline bool isContaining(int i) const { return ((begin_<=i)&&(i<end()));}
+    /** compare this range with range @c I
+     *  @param I the Index to compare
+     *  @return @c true if the range are equals, @c false otherwise
+     **/
+    template<int OtherSize_>
+    inline bool operator==(TRange<OtherSize_> const& I) const
+    { return ((begin_ == I.begin()) && (end() == I.end()));}
+    /** compare this range with range @c I
+     *  @param I the Index to compare
+     *  @return @c true if the range are different, @c false otherwise
+     **/
+    template<int OtherSize_>
+    inline bool operator!=(TRange<OtherSize_> const& I) const
+    { return ((begin_ != I.begin()) || (end() != I.end()));}
+
+  private:
+    int begin_;    ///< First index
+};
+
+/** @ingroup Arithmetic
+ *  @brief Partial Specialization for TRange.
+ *  NA (not available) numbers is part of the TRange.
+ */
+template<int Size_>
+struct Arithmetic< TRange<Size_> > : public std::numeric_limits< TRange<Size_> >
+{
+  /** Adding a Non Available (NA) special number. */
+  static inline  TRange<Size_> NA() throw()
+  { return TRange<Size_>(std::numeric_limits<int>::min(), std::numeric_limits<int>::min(), 0);}
+  /** True if the type has a representation for a "Not Available". */
+  static const bool hasNA = true;
+  /** Test if x is a Non Available (NA) special number
+   *  @param x the Binary number to test.
+   **/
+  static inline bool isNA(TRange<Size_> const& x) throw()
+  { return (x.begin() == std::numeric_limits<int>::min());}
+  /** test if x is  infinite.
+   *  @param x the Binary number to test.
+   **/
+  static inline bool isInfinite(TRange<Size_> const& x) throw() { return false; }
+  /** test if x is  finite.
+   *  @param x the Binary number to test.
+   **/
+  static inline bool isFinite(TRange<Size_> const& x) throw() { return (!isNA(x) && !isInfinite(x));}
+};
+
+/** @ingroup RTTI
+ *  @brief Partial Specialization of the IdTypeImpl for the Type TRange.
+ **/
+template<int Size_>
+struct IdTypeImpl< TRange<Size_> >
+{
+  /** @return the IdType of the type TRange. */
+  static inline Base::IdType returnType() { return(Base::range_);}
+};
+
 /** @ingroup STKernel
  *  @brief Index sub-vector region: Specialization when the size is unknown.
  *
@@ -239,7 +418,34 @@ class TRange<UnknownSize>
      *  @param is the input stream
      *  @param I the range to set
      **/
-    friend istream& operator>> (istream& is, Range& I);
+    friend inline istream& operator>> (istream& is, Range& I)
+    {
+      String str;
+      std::getline(is, str, _T(':'));
+      // get first number
+      if (!stringToType(I.begin_, str))
+      {
+        I = Arithmetic<Range>::NA();
+        is.clear(); is.setstate(std::ios::failbit);
+        return is;
+      }
+      // check if the istream is exhausted
+      if (is.eof())
+      {
+        I.end_ = I.begin_+1;
+        return is;
+      }
+      // skip the current char ":"
+      is.peek();
+      // get second number
+      if ((is >> I.end_).fail())
+      {
+        I = Arithmetic<Range>::NA();
+        is.clear(); is.setstate(std::ios::failbit);
+        return is;
+      }
+      return is;
+    }
 
   private:
     int begin_;    ///< starting index
@@ -248,190 +454,6 @@ class TRange<UnknownSize>
 
 };
 
-/** @ingroup STKernel
- *  @brief Index sub-vector region with fixed size.
- *
- *  A Range is an ordered pair [first,last] denoting a sub-vector
- *  region, similar to a Fortran 90 or Matlab colon notation.
- *  For example :
- *  @code
- *  Vector A(10), B(Range(0,20));
- *  Range I(2,4, false);
- *  A(I) = B(Range(0,2, false));
- *  @endcode
- *  overwrite the elements (2, 3, 4) of A by the elements (0, 1, 2) of B.
- *  There is no stride argument, only contiguous regions are allowed.
- **/
-template<int Size_>
-class TRange
-{
-  public:
-    /** Default constructor. Null range.*/
-    inline TRange() : begin_(baseIdx) {}
-    /** constructor. The first index is defined by the baseIdx macro.
-     **/
-    inline TRange( int) : begin_(baseIdx) {}
-    /** Complete constructor. Give the beginning and the size of the sub-region.
-     *  @param first is the beginning of the sub-region
-     **/
-    inline TRange( int first, int) : begin_(first) {}
-    /** Complete constructor. Give the beginning and the end of the sub-region.
-     * @param first is the beginning of the sub-region
-     **/
-    inline TRange( int first, int, bool) : begin_(first) {}
-    /** Create a copy of an existing TRange.
-     *  @param I The index to copy
-     **/
-    template<int OtherSize_>
-    inline TRange(TRange<OtherSize_> const& I) : begin_(I.begin()) {}
-    /** destructor. */
-    inline ~TRange() {}
-    /** get the first index of the TRange.
-     *  @return the first index of the range
-     **/
-    inline int begin()  const { return begin_;};
-    /** get the ending index of the TRange.
-     *  @return the first index of the range
-     **/
-    inline int end()  const { return begin_ + Size_;};
-    /** get the size of the TRange (the number of elements).
-     *  @return the size of the range
-     **/
-    inline int size()   const { return Size_;};
-    // backward compatibility
-    /** get the first index of the TRange.
-     *  @return the first index of the range
-     **/
-    inline int firstIdx()  const { return begin_;};
-    /** get the last index of the TRange.
-     *  @return the last index of the range
-     **/
-    inline int lastIdx() const { return begin_ + Size_ -1;};
-    /** check if the range is empty or not.
-     *  @return @c true if size <=0, @c false otherwise
-     */
-    inline bool empty() const { return Size_<=0;};
-
-    /** Shift the TRange giving the first element.
-     *  @param first new value of the first element. */
-    inline TRange& shift(int first) { return inc(first - begin_);}
-    /** create the TRange [begin_+inc, end_+inc_].
-     *  @param inc the increment to apply
-     **/
-    inline TRange& inc(int inc) { begin_ +=inc; return *this;}
-    /** create the TRange [begin_+inc, end_].
-     *  @param inc the increment to apply
-     **/
-    inline TRange& incFirst(int inc) { begin_ +=inc; return *this;}
-    /** create the TRange [begin_, end_+inc]
-     * @param inc the increment to apply
-     **/
-    inline TRange& incLast(int inc) { begin_ +=inc; return *this;}
-    /** create the TRange [begin_-dec, end_-dec]
-     *  @param dec the decrement to apply
-     **/
-    inline TRange& dec(int dec) { begin_ -=dec; return *this;}
-    /** @brief create the TRange [begin_-dec, end_]
-     * @param dec the decrement to apply
-     **/
-    inline TRange& decFirst(int dec) { begin_ -=dec;  return *this;}
-    /** create the TRange [begin_, end_-dec]
-     * @param dec the decrement to apply
-     **/
-    inline TRange& decLast(int dec){ begin_ -=dec; return *this;}
-    /** @return this %TRange incremented **/
-    inline TRange& operator+=(int inc) { begin_ += inc; return *this;}
-    /** @return this %TRange decremented **/
-    inline TRange& operator-=(int dec) { begin_ -= dec; return *this;}
-    /** @return this %TRange incremented **/
-    inline TRange operator+(int inc) const { return(TRange(begin_+inc, Size_));}
-    /** @return %TRange I incremented **/
-    inline friend TRange operator+(int inc, TRange const &I) { return I+inc;}
-    /** @return this %TRange decremented **/
-    inline TRange operator-(int dec) const { return(TRange(begin_-dec, Size_));}
-
-    /** check if this TRange in include in an other TRange
-     *  @param I the index to compare
-     *  @return @c true if this is include in @c I, @c false otherwise
-     **/
-    template<int OtherSize_>
-    inline bool isIn(TRange<OtherSize_> const& I) const
-    { return ((begin_>= I.begin())&&(end()<=I.end()));}
-    /** check if the TRange I is include in the this TRange
-     *  @param I the range to compare
-     *  @return @c true if this contain I, @c false otherwise
-     **/
-    template<int OtherSize_>
-    inline bool isContaining(TRange<OtherSize_> const& I) const
-    { return ((begin_<= I.begin())&&(end()>=I.end()));}
-    /** Return true if i is in this TRange
-     *  @param i the integer to compare
-     *  @return @c true if i is in this, @c false otherwise
-     **/
-    inline bool isContaining(int i) const { return ((begin_<=i)&&(i<end()));}
-    /** compare this range with range @c I
-     *  @param I the Index to compare
-     *  @return @c true if the range are equals, @c false otherwise
-     **/
-    template<int OtherSize_>
-    inline bool operator==(TRange<OtherSize_> const& I) const
-    { return ((begin_ == I.begin()) && (end() == I.end()));}
-    /** compare this range with range @c I
-     *  @param I the Index to compare
-     *  @return @c true if the range are different, @c false otherwise
-     **/
-    template<int OtherSize_>
-    inline bool operator!=(TRange<OtherSize_> const& I) const
-    { return ((begin_ != I.begin()) || (end() != I.end()));}
-
-  private:
-    int begin_;    ///< First index
-};
-
-/** @ingroup Arithmetic
- *  @brief Partial Specialization for TRange.
- *  NA (not available) numbers is part of the TRange.
- */
-template<int Size_>
-struct Arithmetic< TRange<Size_> > : public std::numeric_limits< TRange<Size_> >
-{
-  /** Adding a Non Available (NA) special number. */
-  static inline  TRange<Size_> NA() throw()
-  { return TRange<Size_>(std::numeric_limits<int>::min(), std::numeric_limits<int>::min(), 0);}
-  /** True if the type has a representation for a "Not Available". */
-  static const bool hasNA = true;
-  /** Test if x is a Non Available (NA) special number
-   *  @param x the Binary number to test.
-   **/
-  static inline bool isNA(TRange<Size_> const& x) throw()
-  { return (x.begin() == std::numeric_limits<int>::min());}
-  /** test if x is  infinite.
-   *  @param x the Binary number to test.
-   **/
-  static inline bool isInfinite(TRange<Size_> const& x) throw() { return false; }
-  /** test if x is  finite.
-   *  @param x the Binary number to test.
-   **/
-  static inline bool isFinite(TRange<Size_> const& x) throw() { return (!isNA(x) && !isInfinite(x));}
-};
-
-/** @ingroup RTTI
- *  @brief Partial Specialization of the IdTypeImpl for the Type TRange.
- **/
-template<int Size_>
-struct IdTypeImpl< TRange<Size_> >
-{
-  /** @return the IdType of the type TRange. */
-  static inline Base::IdType returnType() { return(Base::range_);}
-};
-
-
-} // namespace STK
-
-#include <map>
-
-namespace STK
-{
 /** @brief Write a TRange in the form first:last (MATLAB-like form) in an
  *  output stream.
  *  @param os output stream
