@@ -135,7 +135,7 @@ class JointBernoulliModel : public IMultiStatModel<Array, WColVector, JointBerno
     virtual Real computeLnLikelihood( RowVector const& rowData) const
     {
       Real sum =0.;
-      for (Integer j= rowData.begin(); j <= rowData.lastIdx(); ++j)
+      for (Integer j= rowData.begin(); j < rowData.end(); ++j)
       {
         sum += rowData[j] * p_param()->lnProb(j)
              + (1-rowData[j] * p_param()->ln1mProb(j) );
